/*************************************
*         Copyright 2015 EDF         *
*************************************/
#pragma once

class CSystem;
class IMonitored;
class CTransition;
class CTrClasses;
class CSeqClasses;

#include "Interfaces.h"
#include <vector>
#include <string>

/**Classe de base d'analyse des rsultats d'une simulation Pycatshoo.
Cette classe propose des fonctions de base permettant l'analyse des rsultats, mais elle peut tre spcialise pour en tendre les possibilits.
*/
class PYC_PUBLIC CAnalyser
{
private:
	CSystem&m_System;
	bool m_LocalSystem;
	IFilterFct*m_SeqFilter;//!<Filtre de squences
	CTrClasses&m_TrClasses;//!<Regroupement des transitions dans des classes
	CSeqClasses&m_SeqClasses;//!<Regroupement des squences dans des classes
	static float m_ConditionLimit;//!<Limite utilise pour le calcul des fractiles
	static bool condition(float value){return value>m_ConditionLimit;}
public:
	/**Construit un analyseur de rsultats pour le systme system.
	Le systme peut tre vide lors de la cration de l'analyseur.
	@param system systme dont les rsultats vont tre analyss
	*/
	explicit CAnalyser(CSystem*system=0);
	~CAnalyser(void);
	/**Retourne le nom complet des diffrents lments monitors.
	@param type type des lments monitors recherchs parmi (PRT : port, ST : tat, TR : transition, IND : indicateur, NULL : tous)
	*/
	std::vector<IMonitored const*>monitoredElts(char const*type=NULL)const;
	/**Retourne l'lment monitor de nom name.
	@param name nom de l'lment cherch
	@param type type de l'lment recherch parmi (PRT : port, ST : tat, TR : transition, IND : indicateur, NULL : indiffrent)
	*/
	IMonitored const*monitoredElt(char const*name,char const*type=NULL)const;
	/**Retourne le nom des diffrents indicateurs.
	*/
	std::vector<IIndicator*>indicators()const;
	/**Retourne l'indicateur de nom name.
	@param name nom de l'lment cherch
	*/
	IIndicator const*indicator(char const*name)const;
	std::vector<double>instants()const;//!<Retourne la liste des instants de calcul des indicateurs
	/**Renseigne la fonction de filtrage des squences sous la forme d'un foncteur boolen d'une squence*/
	void setSeqFilter(IFilterFct*seqFilter);
	/**Renseigne la fonction de filtrage des squences sous la forme d'une fonction boolenne d'une squence.*/
	void setSeqFilter(bool (*seqFilter)(ISequence const&));
	/**Renseigne la fonction de filtrage des squences sous la forme d'un pattern de dfinition d'un filtre*/
	void setSeqFilter(char const*filterPattern);
	/**Retourne la liste des squences filtres*/
	std::vector<ISequence*>filteredSeq()const;
	/**Charge un fichier rsultat
	@param resFile chemin du fichier de rsultats
	*/
	void loadResults(char const*resFile);
	/**Retourne dans values toutes les valeurs des diffrentes squences  la date t pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param t date  laquelle les valeurs doivent tre obtenues
	@return vecteur des valeurs rsultat de la requte
	*/
	std::vector<float>allValues(char const*name,double t)const;
	/**Retourne dans values toutes les valeurs de la squence i pour l'lment monitor de nom name (le nom doit tre complet).
	@param seq squence  explorer
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts vecteur de stockage des dates des changements de valeurs
	@param values vecteur de stockage des valeurs
	*/
	void allSeqValues(ISequence const*seq,char const*name,std::vector<double>&ts,std::vector<float>&values)const;
	/**Retourne dans nbs le nombre de squences pour lesquelles l'lment monitor de nom name (le nom doit tre complet) satisfait la condition aux instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@param condition condition de calcul du temps de rsidence exprime sous forme d'un foncteur boolen de la valeur de l'lment
	@return vecteur rsultat des nombres d'occurrences de la condition aux diffrents instants (autant de valeurs que d'lments dans ts)
	*/
	std::vector<SEQ_ID>occurrences(char const*name,std::vector<double>const&ts,ICondFctFloat const&condition)const;
	/**Retourne dans nbs le nombre de squences pour lesquelles l'lment monitor de nom name (le nom doit tre complet) satisfait la condition aux instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@param condition condition de calcul du temps de rsidence exprime sous forme d'une fonction boolenne de la valeur de l'lment
	@return vecteur rsultat des nombres d'occurrences de la condition aux diffrents instants (autant de valeurs que d'lments dans ts)
	*/
	std::vector<SEQ_ID>occurrences(char const*name,std::vector<double>const&ts,bool (*condition)(float))const{return occurrences(name,ts,CCondFctFloat(condition));}
	/**Calcule la probabilit que l'lment monitor de nom name (le nom doit tre complet) satisfasse la condition avant les instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates limites avant lesquelles la condition doit tre satisfaite
	@param oprateur de comparaison
	@param valeur de comparaison
	@param pr vecteur rsultat des probabilits de ralisation de la condition avant les instants (autant de valeurs que d'lments dans ts)
	*/
	std::vector<float>realized(char const*name,std::vector<double>const&ts,char const*op,float value)const{return realized(name,ts,CCondCompFloat(op,value));}
	/**Calcule la probabilit que l'lment monitor de nom name (le nom doit tre complet) satisfasse la condition avant les instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param condition condition de calcul du temps de rsidence exprime sous forme d'un foncteur boolen de la valeur de l'lment
	@param ts dates limites avant lesquelles la condition doit tre satisfaite
	@return vecteur rsultat des probabilits de ralisation de la condition avant les instants (autant de valeurs que d'lments dans ts)
	*/
	std::vector<float>realized(char const*name,std::vector<double>const&ts,ICondFctFloat const&condition)const;
	/**Calcule la probabilit que l'lment monitor de nom name (le nom doit tre complet) satisfasse la condition avant les instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param condition condition de calcul du temps de rsidence exprime sous forme d'une fonction boolenne de la valeur de l'lment
	@param ts dates limites avant lesquelles la condition doit tre satisfaite
	@param pr vecteur rsultat des probabilits de ralisation de la condition avant les instants (autant de valeurs que d'lments dans ts)
	*/
	std::vector<float>realized(char const*name,std::vector<double>const&ts,bool (*condition)(float))const{return realized(name,ts,CCondFctFloat(condition));}
	/**Retourne le temps de sjour moyen entre it et ft pendant lequel l'lment monitor de nom name (le nom doit tre complet) satisfait la condition.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param it temps initial du calcul
	@param ft temps final du calcul
	@param condition condition de calcul du temps de rsidence exprime sous forme d'un foncteur boolen de la valeur de l'lment
	*/
	float residenceTime(char const*name,double it,double ft,ICondFctFloat const&condition)const;
	/**Retourne le temps de sjour moyen entre it et ft pendant lequel l'lment monitor de nom name (le nom doit tre complet) satisfait la condition.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param it temps initial du calcul
	@param ft temps final du calcul
	@param condition condition de calcul du temps de rsidence exprime sous forme d'une fonction boolenne de la valeur de l'lment
	*/
	float residenceTime(char const*name,double it,double ft,bool (*condition)(float))const{return residenceTime(name,it,ft,CCondFctFloat(condition));}
	/**Retourne dans resTs le temps de sjour moyen de l'lment monitor de nom name (le nom doit tre complet) entre les bornes dfinies par lims.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param it date de dbut de calcul de la distribution
	@param ft date de fin de calcul de la distribution
	@param lims limites des intervalles de calcul de la distribution dans l'ordre croissant (si toutes les limites sont nulles, le tableau est rempli automatiquement)
	@return vecteur de stockage de la distribution (une valeur de plus que dans lims)
	*/
	std::vector<double>residenceTimes(char const*name,double it,double ft,std::vector<float>const&lims)const;
	/**Calcule les valeurs extrmes de l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param min valeur minimale prise par l'lment
	@param max valeur maximale prise par l'lment
	*/
	void extremeValues(char const*name,float&min,float&max)const;
	/**Calcule les valeurs extrmes de l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@return vecteur[valeur minimale,valeur maximale]
	*/
	std::vector<float>extremeValues(char const*name)const{std::vector<float>v(2,(float)HUGE_VAL);extremeValues(name,v[0]=(float)-HUGE_VAL,v[1]);return v;}
	/**Retourne la valeur de l'lment monitor de nom name (le nom doit tre complet) telle que la probabilit d'tre au dessus  l'instant t est gal  pct.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param t instant de calcul
	@param pct dfinition du fractile
	*/
	float quantileGt(char const*name,float t,float pct)const;
	/**Retourne les valeurs de l'lment monitor de nom name (le nom doit tre complet) telles que la probabilit d'tre au dessus aux instants ts est gal  pct.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts instants de calcul
	@param pct dfinition du fractile
	*/
	std::vector<float> quantilesGt(char const*name, std::vector<double>const & ts, float pct)const;
	/**Retourne la valeur de l'lment monitor de nom name (le nom doit tre complet) telle que la probabilit d'tre au dessous  l'instant t est gal  pct.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param t instant de calcul
	@param pct dfinition du fractile
	*/
	float quantileLe(char const*name,float t,float pct)const;
	/**Retourne les valeurs de l'lment monitor de nom name (le nom doit tre complet) telles que la probabilit d'tre au dessous aux instants ts est gal  pct.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts instants de calcul
	@param pct dfinition du fractile
	*/
	std::vector<float> quantilesLe(char const*name, std::vector<double>const & ts, float pct)const;
	/**Retourne la valeur de l'lment monitor de nom name (le nom doit tre complet) telle que le temps moyen pass au dessus est gal  pct.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param pct dfinition du fractile : pourcentage du temps de simulation
	@param precis cart maximum tolr entre le rsultat et la valeur exact (le fractile est estim par dichotomie)
	*/
	float quantile(char const*name,float pct,float precis)const;
	/**Retourne les  fractiles min et max pour les instants du vecteur temps pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts vecteur temps pour le calcul des fractiles
	@param quantilesMin vecteur des quantiles minimaux (sortant)
	@param pctMin niveaux des quantiles minimaux demands
	@param quantilesMax vecteur des quantiles maximaux (sortant)
	@param pctMax niveaux des quantiles maximaux demands
	*/
	void quantiles(char const*name, std::vector<double>const & ts, std::vector<float> & quantilesMin, float pctMin, std::vector<float> & quantilesMax, float pctMax)const;
	/**Retourne la moyenne de la valeur de l'lment monitor de nom name (le nom doit tre complet) entre les temps it et ft.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param it instant initial de calcul
	@param ft instant final de calcul
	@return valeur moyenne calcule
	*/
	float meanValue(char const*name,double it,double ft)const;//Moyenne globale entre les temps it et ft
	/**Retourne dans mean et stdDev la moyenne et l'cart type  l'instant t pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param t date  laquelle les calculs doivent tre faits
	@param mean moyenne rsultat de l'analyse  la date t
	@param stdDev cart type rsultat de l'analyse  la date t
	*/
	void meanValue(char const*name,double t,float&mean,float&stdDev)const;//Moyenne et cart type  l'instant t
	/**Retourne dans means et stdDevs les moyennes et les carts types aux instants ts pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@param means vecteur de stockage des moyennes (autant de valeurs que de points dans ts)
	@param stdDevs vecteur de stockage des carts type (autant de valeurs que de points dans ts)
	*/
	void meanValues(char const*name,std::vector<double>const&ts,std::vector<float>&means,std::vector<float>&stdDevs)const;
	/**Retourne les moyennes aux instants ts pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@return vecteur de stockage des moyennes (autant de valeurs que de points dans ts)
	*/
	std::vector<float>meanValues(char const*name,std::vector<double>const&ts)const;
	/**Retourne les intervalles de confiance aux instants ts pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@param pct pourcentage de confiance de l'intervalle
	@return vecteur de stockage des intervalles de confiance (autant de valeurs que de points dans ts)
	*/
	std::vector<float>confIntValues(char const*name,std::vector<double>const&ts,float pct)const;
	/**Retourne les carts types aux instants ts pour l'lment monitor de nom name (le nom doit tre complet).
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates auxquelles les calculs doivent tre faits
	@return vecteur de stockage des carts type (autant de valeurs que de points dans ts)
	*/
	std::vector<float>stdDevValues(char const*name,std::vector<double>const&ts)const;
	/**Retourne dans la distribution de la valeur de l'lment monitor de nom name (le nom doit tre complet)  l'instant t.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param t date de calcul de la distribution
	@param lims limites des intervalles de calcul de la distribution dans l'ordre croissant (si toutes les limites sont nulles, le tableau est rempli automatiquement)
	@return vecteur de stockage de la distribution (une valeur de plus que dans lims)
	*/
	std::vector<float>distribution(char const*name,double t,std::vector<float>const&lims)const;
	/**Retourne dans la distribution de la valeur de l'lment monitor de nom name (le nom doit tre complet) moyenne sur les instants ts.
	@param name nom de l'lment monitor dont on analyse les valeurs
	@param ts dates de calcul de la distribution
	@param lims limites des intervalles de calcul de la distribution dans l'ordre croissant (si toutes les limites sont nulles, le tableau est rempli automatiquement)
	@return vecteur de stockage de la distribution (une valeur de plus que dans lims)
	*/
	std::vector<float> meanDistribution(char const * name, std::vector<double>const & ts, std::vector<float>const & lims)const;
	/**Enregistre un fichier XML des squences filtres
	@param pct pourcentage des squences filtres restitu
	@param file nom du fichier XML de squences  construire
	@param xsltFile nom du fichier de feuille de style utilis pour construire le fichier HTML correspondant (si NULL, le fichier HTML n'est pas construit)
	*/
	bool printFilteredSeq(float pct,char const*file,char const*xsltFile=NULL)const;
	/**Marque les squences filtres comme  conserver ou non*/
	bool keepFilteredSeq(bool restart);
	/**Excute l'analyse paramtre par le fichier
	@param file nom du fichier de paramtrage de l'analyse
	*/
	bool analyse(char const*file);
	/**Place la transitions dans une classe (ou la sort des classes si la string est vide)*/
	void setTransitionClass(ITransition const*trans,int ind,char const*className);
	/**Supprime une classe de transition*/
	void removeTransitionClass(char const*className){setTransitionClass(NULL,0,className);}
	/**Dfinit une classe de squences par un filtre*/
	void setSequenceClass(char const*className,IFilterFct*filter);
	/**Supprime une classe de squence (toutes si className est NULL)*/
	void removeSequenceClass(char const*className){setSequenceClass(className,NULL);}
};

