/*************************************
*         Copyright 2015 EDF         *
*************************************/
#ifndef COMPONENT_H
#define COMPONENT_H

#include "NamedComp.h"
#include "Interfaces.h"

class CState;
class CMessageBox;
class CTransition;
class CXMLNode;
class CSystem;
class CComponentP;
class CVariable;
class CMainWindow;
class CObjectEditor;

struct TVarType{
	/**Enumr de dfinition du type d'une variable.*/
	typedef enum EVarType{
		t_bool=0,//!<Variable boolenne
		t_int,//!<Variable entire
		t_float,//!<Variable relle
		t_double,//!<Variable relle double prcision
		t_complex,//!<Variable complexe double prcision
		t_string//!<Variable chane de caractres
	}EVarType;
};
/**Classe de gestion des composants.
	Les composants correspondent aux types de la BdC.
	Il est possible d'inclure des composants dans un autre composant (diagramme hirarchique).

	Un composant est un fils d'un systme ou d'un autre composant ; il peut contenir :
	- un ou plusieurs diagrammes d'tats (tats et transitions entre tats),
	- des variables,
	- des rfrences  ces variables,
	- des botes de messages.

	Un composant peut tre muni de mthodes qui peuvent tre dclares comme sensibles aux variations des tats et des variables.
*/
class PYC_PUBLIC CComponent:public CNamedComp
{
	friend class CSystem;
	friend class CSystemP;
	friend class CComponentP;
	friend struct CPycObj;
	CComponentP*m_CompP;
	bool m_bRunning;//Si vrai, le composant a t cr pendant la simulation
	void updatePlanning();
	void updatePlanningInt();
	CState*buildState(char const*state,char const*automate,unsigned index);
	CVariable*buildVariable(char const*,char const*type);
	void activate();//!<Active tous les tats du composant mais pas de ses sous composants.
	void activateAll();//!<Active les tats du composant et de ses sous composants.
	void resetAll();//!<Rinitialise toutes les variables et dsactive les tats du composant et de ses sous composants.
	void deleteData();
protected:
	CComponent(PyObject*pyObj,char const*name,CComponent&parent);
	CComponent(PyObject*pyObj,char const*name,CSystem&system);
	CComponent(PyObject*pyObj,char const*name);
	PyObject*m_Self;//!<Rfrence  l'objet python correspondant (NULL, si c'est un objet purement C++).
	void storeParameter(char const*name,char const*value);
	void updateName();
	void addSysMethod(char const*name,void(CComponent::*method)(),PyObject*pVoid,bool bCheck);
	virtual void deleted(){}//Appele en fin de deleteData pour effectuer un mnage spcifique (singletons Python)
public:
	virtual~CComponent(void);
	CComponent(char const*name,CComponent&parent);
	CComponent(char const*name,CSystem&system);
	explicit CComponent(char const*name);
	CComponentP&compP(){return *m_CompP;}
	CComponentP const&compP()const{return *m_CompP;}
	void deleteCmp();//!<Dtruit le composant sauf si m_Self est non nul (alors, ne dtruit que l'intrieur)
	void rename(char const*newName);//!<Modifie le nom du composant
	std::vector<CComponent*>getComponents(char const*name)const;
	std::vector<IMessageBox*>getMessageBoxes()const;
	std::vector<IVariable*> getVariables()const;
	std::vector<IReference*>getReferences()const;
	std::vector<IAutomaton*>getAutomata()const;
	std::vector<IState*>getStates()const;
	std::vector<ITransition*>getTransitions()const;
	IPDMPManager*getPDMPManager(char const*name)const;//!<Retourne le gestionnaire d'ODE de nom name
	ISLEManager*getSLEManager(char const*name)const;//!<Retourne le gestionnaire de systme d'quations linaires de nom name
	IMILPManager*getMILPManager(char const*name)const;//!<Retourne le gestionnaire d'optimisation sous contraintes de nom name
	/**Retourne l'automate de nom name.*/
	IAutomaton*getAutomaton(char const*name)const;
	IPDMPManager*addPDMPManager(char const*name);//!<Ajoute un gestionnaire d'ODE de nom name s'il n'exsite pas dj
	ISLEManager*addSLEManager(char const*name);//!<Ajoute un gestionnaire de systme d'quations linaires de nom name s'il n'existe pas dj
	IMILPManager*addMILPManager(char const*name);//!<Ajoute un gestionnaire d'optimisation sous contraintes de nom name s'il n'existe pas dj
	/**Ajoute une mthode de contrle du paramtrage du composant appele en dbut de simulation*/
	template<typename Type>void addCheckMethod(char const*name,void(Type::*method)()){addSysMethod(name,static_cast<void(CComponent::*)()>(method),NULL,true);}
	/**Ajoute une mthode de dmarrage du composant (appele en dbut de chaque squence*/
	template<typename Type>void addStartMethod(char const*name,void(Type::*method)()){addSysMethod(name,static_cast<void(CComponent::*)()>(method),NULL,false);}
	/**Ajoute une tape de nom name.
	La mthode retourne l'tape ajoute ou retrouve dans la liste des tapes.*/
	IStep*addStep(char const*name);
	/**Ajoute une mthode de nom methodName  l'tape de nom stepName.*/
	void addStepMethod(char const*stepName,char const*methodName);
	/**Construit un automate de nom name
	@param name nom de l'automate  ajouter
	@return l'automate construit
	*/
	IAutomaton*addAutomaton(char const*name);
	/**Retourne le nom de la classe du composant*/
	virtual char const*className()const;
	/**Retourne le sous composant de nom name ventuellement compos.*/
	CComponent*getComponent(char const*name,bool check=true)const;
	/**Ajoute une quation  rsoudre.
	@param systemName nom du systme d'quations linaires  modifier
	@param name nom de l'quation  ajouter
	@param equation mthode de dfinition de l'quation
	*/
	template<typename Type>void addSLEEquation(char const*systemName,char const*name,void (Type::*equation)(IEquation&)){
		getSLEManager(systemName)->addEquation(name,*this,static_cast<void(CNamedComp::*)(IEquation&)>(equation));
	}
	void addPySLEEquation(char const*systemName,char const*equation){getSLEManager(systemName)->addPyEquation(equation,*this);}
	void addPySLEEquation(char const*systemName,char const*equation,PyObject*pyObj){getSLEManager(systemName)->addPyEquation(equation,*this,pyObj);}
	/**Ajoute une variable au systme linaire de nom name.
	@param systemName nom du systme d'quations linaires  modifier
	@param v variable  ajouter dans le systme
	*/
	void addSLEVariable(char const*systemName,IVariable&v){getSLEManager(systemName)->addVariable(v);}
	/**Ajoute une inquation  rsoudre.
	@param systemName nom de l'optimiseur sous contraintes  modifier
	@param name nom de l'inquation  ajouter
	@param inequation mthode de dfinition de l'inquation
	*/
	template<typename Type>void addMILPInequality(char const*systemName,char const*name,void (Type::*inequality)(IInequality&)){
		getMILPManager(systemName)->addInequality(name,*this,static_cast<void(CComponent::*)(IInequality&)>(inequality));
	}
	void addPyMILPInequality(char const*systemName,char const*equation){getMILPManager(systemName)->addPyInequality(equation,*this);}
	void addPyMILPInequality(char const*systemName,char const*equation,PyObject*pyObj){getMILPManager(systemName)->addPyInequality(equation,*this,pyObj);}
	/**Ajoute une variable  l'optimiseur sous contraintes de nom name.
	@param systemName nom de l'optimiseur sous contraintes  modifier
	@param v variable  ajouter au systme
	@param co coefficient de la fonction objectif associ  la variable
	*/
	void addMILPVariable(char const*systemName,IVariable&v,double co){getMILPManager(systemName)->addVariable(v,co);}
	void addMILPVariable(char const*systemName,IVariable&v){getMILPManager(systemName)->addVariable(v);}
	/**Ajoute une mthode de calcul des quations diffrentielles et/ou des variables calcules.
	@param systemName nom du systme d'ODE  modifier
	@param name nom de la mthode de calcul
	@param odeFct mthode de calcul
	@param order numro d'ordre d'appel de la mthode
	*/
	template<typename Type>void addPDMPEquationMethod(char const*systemName,char const*name,void(Type::*odeFct)(),int order = 0){
		getPDMPManager(systemName)->addEquationMethod(name,*this,static_cast<void(CComponent::*)()>(odeFct),order);
	}
	void addPyPDMPEquationMethod(char const*systemName,char const*ode,PyObject*pyObj,int order = 0){getPDMPManager(systemName)->addPyEquationMethod(ode,*this,pyObj,order);}
	/**Ajoute une variable comme lment de l'tat de l'ODE de nom name.
	@param systemName nom du systme d'ODE  modifier
	@param v variable gre par le systme d'ODE.
	*/
	void addPDMPODEVariable(char const*systemName,IVariable&v);
	/**Ajoute une variable comme lment calcul au cours de l'ODE.
	@param systemName nom du systme d'ODE  modifier
	@param v variable calcule  ajouter
	*/
	void addPDMPExplicitVariable(char const*systemName,IVariable&v);
	/**Ajoute une condition  l'ODE.
	@param systemName nom du systme d'ODE  modifier
	@param name nom de la mthode  utiliser comme condition
	@param condFct mthode condition (doit retourner un double)
	*/
	template<typename Type>void addPDMPCondition(char const*systemName,char const*name,double(Type::*condFct)()){
		getPDMPManager(systemName)->addCondition(name,*this,static_cast<double(CComponent::*)()>(condFct));
	}
	template<typename Type>void addPDMPBoundaryCheckerMethod(char const*systemName,char const*name,double(Type::*condFct)()){
		getPDMPManager(systemName)->addCondition(name,*this,static_cast<double(CComponent::*)()>(condFct));
	}
	void addPyPDMPCondition(char const*systemName,char const*condition,PyObject*pyObj){getPDMPManager(systemName)->addPyCondition(condition,*this,pyObj);}
	void addPyPDMPCondition(char const*systemName,char const*condition){addPyPDMPCondition(systemName,condition,NULL);}
	/**Ajoute un condition  l'ODE sous la forme de la condition d'une transition*/
	void addPDMPWatchedTransition(char const*systemName,ITransition&trans){getPDMPManager(systemName)->addWatchedTransition(trans);}
	void addPDMPWatchedTransition(char const*systemName,char const*trans){getPDMPManager(systemName)->addWatchedTransition(*getTransition(trans));}
	/**Dfinit une fonction appele avant l'intgration de l'ODE.
	@param systemName nom du systme d'ODE  modifier
	@param name nom de la mthode  appeler en dbut d'intgration
	@param beginFct mthode  appeler
	*/
	template<typename Type>void addPDMPBeginMethod(char const*systemName,char const*name,void(Type::*beginFct)()){
		getPDMPManager(systemName)->addBeginMethod(name,*this,static_cast<void(CComponent::*)()>(beginFct));
	}
	void addPyPDMPBeginMethod(char const*systemName,char const*fct,PyObject*pyObj){getPDMPManager(systemName)->addPyBeginMethod(fct,*this,pyObj);}
	void addPyPDMPBeginMethod(char const*systemName,char const*fct){getPDMPManager(systemName)->addPyBeginMethod(fct,*this);}
	/**Dfinit une fonction appele aprs l'intgration de l'ODE.
	@param systemName nom du systme d'ODE  modifier
	@param name nom de la mthode  appeler en fin d'intgration
	@param endFct mthode  appeler
	*/
	template<typename Type>void addPDMPEndMethod(char const*systemName,char const*name,void(Type::*endFct)(char const*)){
		getPDMPManager(systemName)->addEndMethod(name,*this,static_cast<void(CComponent::*)(char const*)>(endFct));
	}
	void addPyPDMPEndMethod(char const*systemName,char const*fct){getPDMPManager(systemName)->addPyEndMethod(fct,*this);}
	void addPyPDMPEndMethod(char const*systemName,char const*fct,PyObject*pyObj){getPDMPManager(systemName)->addPyEndMethod(fct,*this,pyObj);}
	/**Ajoute une bote de message.
	@param name nom de la bote de messages  ajouter
	@return la bote de messages construite
	*/
	IMessageBox*addMessageBox(char const*name);
	/**Ajoute une variable  la bote de messages.
	@param name nom de la bote de messages
	@param v variable  ajouter
	@param alias nom sous lequel la variable doit tre vue lors de la connexion  cette bote de messages
	*/
	void addMessageBoxExport(char const*name,IVarBase*v,char const*alias=NULL);
	/**Ajoute une rfrence  la bote de message et prcise ne nom de la variable  y connecter.
	@param name nom de la bote de messages
	@param r rfrence  ajouter
	@param varName nom ou alias de la variable  connecter  la rfrence
	*/
	void addMessageBoxImport(char const*name,IReference*r,char const*varName);
	/**Ajoute une rfrence  la bote de message et prcise ne nom de la variable  y connecter de manire optionnelle ainsi que la valeur  attribuer en l'absence de connexion.
	@param name nom de la bote de messages
	@param r rfrence  ajouter
	@param type type de la valeur parmi TValueType
	@param defaultValue valeur par dfaut attribue en l'abscence de connexion
	@param varName nom ou alias de la variable  connecter  la rfrence
	*/
	void addMessageBoxOptionalImport(char const*name,IReference*r,int type, double defaultValue,char const*varName);
	/**Ajoute une variable, dfinit son type et sa valeur initiale.
	@param name nom de la nouvelle variable
	@param type type de la nouvelle variable
	@param val valeur initiale de la nouvelle variable (sous forme de double)
	@return la variable construite
	*/
	IVariable*addVariable(char const*name,TVarType::EVarType type,double val,bool bReinitialized=false);
	//Pour limiter les ambiguits avec addVariable initialise avec un char*
	IVariable*addVariable(char const*name,TVarType::EVarType type,float val,bool bReinitialized=false){return addVariable(name,type,(double)val,bReinitialized);}
	IVariable*addVariable(char const*name,TVarType::EVarType type,int val,bool bReinitialized=false){return addVariable(name,type,(double)val,bReinitialized);}
	IVariable*addVariable(char const*name,TVarType::EVarType type,bool val,bool bReinitialized=false){return addVariable(name,type,(double)val,bReinitialized);}
	/**Ajoute une variable, dfinit son type et sa valeur initiale.
	@param name nom de la nouvelle variable
	@param type type de la nouvelle variable
	@param val valeur initiale de la nouvelle variable (sous forme de chane de caractres)
	@return la variable construite
	*/
	IVariable*addVariable(char const*name,TVarType::EVarType type,char const*val,bool bReinitialized=false);
	/**Retourne la variable de nom name (le nom peut tre compos pour accder aux variables des sous composants).*/
	IVariable*getVariable(char const*name)const;
	/**Ajoute une rfrence  des variable.
	@param name nom de la nouvelle rfrence
	@return la rfrence construite
	*/
	IReference*addReference(char const*name);
	/**Retourne la rfrence de nom name (le nom peut tre compos pour accder aux rfrences des sous composants).*/
	IReference*getReference(char const*name)const;
	/**Retourne la transition de nom name (le nom peut tre compos pour accder aux transitions des sous composants).*/
	ITransition*getTransition(char const*name)const;
	/**Retourne le bote de message de nom name (le nom peut tre compos pour accder aux botes des sous composants).*/
	IMessageBox*getMessageBox(char const*name)const;
	/**Ajoute une transition de sortie de l'tat start.
	@param name nom de la nouvelle transition
	@param start nom de l'tat de dpart de la transition
	@return la transition construite
	*/
	ITransition*addTransition(char const*name,char const*start);
	/**Ajoute une transition de sortie de l'tat start.
	@param name nom de la nouvelle transition
	@param start tat de dpart de la transition
	@return la transition construite
	*/
	ITransition*addTransition(char const*name,CState*start);
	/**Ajoute un tat dans un automate.
	@param automatonName nom de l'automate
	@param name nom du nouvel tat
	@param index indice de l'tat (si ngatif, l'indice est gal au nombre d'tats dj dans l'automate)
	@return l'tat construit
	*/
	IState*addState(char const*automatonName,char const*name,int index);
	/**Ajoute un tat.
	@param name nom du nouvel tat
	@return l'tat construit
	*/
	IState*addState(char const*name);
	/**Retourne l'tat de nom name (le nom peut tre compos pour accder aux tats des sous composants).*/
	IState*getState(char const*name)const;
	/**Dfinit l'tat de nom name comme tat initialement actif.*/
	void setInitState(char const*name){setInitState(getState(name));}
	/**Dfinit l'tat comme tat initialement actif.*/
	void setInitState(IState*st);
	/**Retourne vrai si l'tat de nom name est initialement actif.*/
	bool isInitState(char const*name)const;
	/**Rensigne un paramtre quelconque qui doit tre interprt  partir de la chane de caractres value.*/
	virtual void setParameter(const char*name,const char*value);
	/**Verifie le composant avant simulation*/
	bool verify();
};

#endif
