/*************************************
*         Copyright 2015 EDF         *
*************************************/
#ifndef INTERFACES_H
#define INTERFACES_H

#include "ILogManager.h"
#include "NamedComp.h"

#include <limits.h>
#include <vector>
#include <complex>
#include <typeinfo>

class CSystem;
class CSystemP;
class IVariable;
class ITransition;
class IState;
class IEquation;
class IIndicator;
class IInequality;
class IAutomaton;
class CMessageBox;
class CComponent;
class CVariable;
class CTransition;

/**Interface de base d'un moniteur.
	Cette classe n'est drive que par la classe CSysteme.
	Elle permet aux lments monitors de connatre le moniteur et de lui signaler les changements.
*/
class PYC_PUBLIC IMonitor
{
protected:
	IMonitor(void){}
public:
	/**Demande au moniteur d'enregistrer la valeur de l'lment.*/
	virtual void doMonitor(IMonitored const&elt)=0;
	/**Demande d'ajouter ou d'enlever l'lment du monitoring.*/
	virtual void monitor(IMonitored&elt,bool bMonit)=0;
};

/**Interface de base d'un lment suivi par un moniteur.
	Cette classe est spcialise par CTransition, CState et CVariable.
	L'interface dfinit localement la gestion du moniteur et de la trace.
*/
class PYC_PUBLIC IMonitored{
	friend class CSystemP;
protected:
	IMonitor*m_Monitor;//!<Moniteur de suivi de l'lment.
	int m_Trace;//!<Niveau de trace de l'lment (0 = pas de trace, 1 = trace en fin de cycle, >1 = trace  chaque affectation).
	bool m_AlwaysMonitored;//!<Si vrai, l'lment est monitor en permanence
	int m_Id;//!<Id d'identification de l'lment monitor (dfini de manire unique (si ngatif : fourni et non modifiable))
	IMonitored():m_Monitor(0),m_Trace(0),m_AlwaysMonitored(false),m_Id(0){}
	virtual~IMonitored(){}
public:
	/**Retourne l'Id de l'lment monitor.*/
	unsigned int id()const{return (unsigned)(m_Id>0?m_Id:-m_Id);}
	/**Dsigne le moniteur de suivi de l'lment (si NULL, l'lment n'est pas suivi), renseigne l'id prdfini et modifie le niveau de monitoring.*/
	virtual void setMonitor(IMonitor*monitor,unsigned id,bool always){if(monitor)monitor->monitor(*this,true);else if(m_Monitor)m_Monitor->monitor(*this,false);m_AlwaysMonitored=(monitor && always);m_Id=monitor?-(int)id:0;}
	/**Retourne le moniteur de suivi de l'lment.*/
	virtual IMonitor*monitor()const{return m_Monitor;}
	/**Modifie le niveau de trace de l'lment (si 0 pas de trace de l'lment).*/
	virtual void setTrace(int trace){m_Trace=trace;}
	/**Retourne le niveau de trace de l'lment.*/
	virtual int trace()const{return m_Trace;}
	/**Modifie l'option de monitoring permanent*/
	void setAlwaysMonitored(bool bAlways){m_AlwaysMonitored=bAlways;}
	/**Retourne vrai si l'lment est monitor en permanence*/
	int isAlwaysMonitored()const{return m_AlwaysMonitored;}
	/**Retourne le nom simple de l'lment.*/
	virtual char const*basename()const{return 0;}
	/**Retourne le nom complet de l'lment (incluant la hirarchie des composants propritaires de l'lment).*/
	virtual char const*name()const=0;
	/**Retourne le type d'lment*/
	virtual char const*type()const=0;
	/**Retourne la valeur initiale float de l'lment*/
	virtual float fInitValue()const=0;
	/**Retourne l'objet parent de l'lment (dans Pycatshoo, il s'agit d'un CComponent).*/
	virtual CNamedComp*parent()const=0;
	int compare(IMonitored const&other)const;//!<Comparaison en fonction du type et du nom
	bool operator<(IMonitored const&other)const;//!<Comparaison en fonction du type et du nom
};

/**Class de gestion des diffrentes mthodes passes en paramtres dans Pycatshoo.
La classe permet de grer des mthodes C++ ou des mthodes Python ou encore des fonctions Python sous la forme d'PyObject.*/
class PYC_PUBLIC CMethod{
protected:
	CNamedComp*m_Comp;
	char*m_Basename;
	bool m_Py;
	static bool st_bUseNameComp;//!<Si vrai, la comparaison des mthodes prend en compte le nom des composants, sinon, c'est uniquement l'adresse qui compte
public:
	union{
		double(CNamedComp::*m_DblFct)();
		float(CNamedComp::*m_FltFct)();
		int(CNamedComp::*m_IntFct)();
		bool(CNamedComp::*m_BoolFct)();
		void(CNamedComp::*m_VoidFct)();
		double(CNamedComp::*m_DblFctDbl)(double);
		void(CNamedComp::*m_EndFct)(char const*);
		void(CNamedComp::*m_Equation)(IEquation&);
		void(CNamedComp::*m_Inequality)(IInequality&);
		PyObject*m_PyObj;
	};
	CMethod():m_Comp(NULL),m_Basename(NULL),m_Py(false),m_VoidFct(NULL){}
	CMethod(CNamedComp*comp,char const*name,PyObject*pVoid);
	CMethod(CNamedComp*comp,char const*name,double(CNamedComp::*fct)());
	CMethod(CNamedComp*comp,char const*name,float(CNamedComp::*fct)());
	CMethod(CNamedComp*comp,char const*name,int(CNamedComp::*fct)());
	CMethod(CNamedComp*comp,char const*name,bool(CNamedComp::*fct)());
	CMethod(CNamedComp*comp,char const*name,double(CNamedComp::*fct)(double));
	CMethod(CNamedComp*comp,char const*name,void(CNamedComp::*beginFct)());
	CMethod(CNamedComp*comp,char const*name,void(CNamedComp::*endFct)(char const*));
	CMethod(CNamedComp*comp,char const*name,void(CNamedComp::*equation)(IEquation&));
	CMethod(CNamedComp*comp,char const*name,void(CNamedComp::*inequality)(IInequality&));
	CMethod(CMethod const&other):m_Comp(NULL),m_Basename(NULL),m_Py(false){*this=other;}
	virtual ~CMethod();
	char const*basename()const{return m_Basename;}
	char const*compName()const;
	CNamedComp*comp()const{return m_Comp;}
	CMethod&operator=(CMethod const&other);
	int compare(CMethod const&other)const;//!<Comparaison en fonction du nom
	bool operator<(CMethod const&other)const{return compare(other)<0;}
	bool operator==(CMethod const&other)const{return 0==compare(other);}
	static bool mLess(CMethod const*c1,CMethod const*c2){return c1->compare(*c2)<0;}
	double dValue()const;//!<Retourne la valeur double obtenue par appel de dblFct(void) ou d'une fonction Python quivalente
	virtual float fValue()const;//!<Retourne la valeur float obtenue par appel de fltFct(void) ou d'une fonction Python quivalente
	int iValue()const;//!<Retourne la valeur int obtenue par appel de intFct(void) ou d'une fonction Python quivalente
	virtual bool bValue()const;//!<Retourne la valeur bool obtenue par appel de boolFct(void) ou d'une fonction Python quivalente
	double dValue(double val)const;//!<Retourne la valeur double obtenue par appel de dblFctDbl(double) ou d'une fonction Python quivalente
	void call()const;//!<Appelle la fonction voidFct(void) ou la fonction Python quivalente
	void call(char const*)const;//!<Appelle la fonction endFct(void) ou la fonction Python quivalente
	void call(IEquation&)const;//!<Appelle la fonction equation(IEquation) ou la fonction Python quivalente
	void call(IInequality&)const;//!<Appelle la fonction inequality(IInequality) ou la fonction Python quivalente
	static void setUseNameComp(bool bUseCompName);
};

struct SSensitiveMethod:CMethod{
	char m_Sens;
	SSensitiveMethod():m_Sens(0){}
	SSensitiveMethod(CNamedComp*comp,char const*name,void(CNamedComp::*method)(),int sens):CMethod(comp,name,method),m_Sens(sens){}
	SSensitiveMethod(CNamedComp*comp,char const*name,PyObject*pyObj,int sens):CMethod(comp,name,pyObj),m_Sens(sens){}
};

struct TOperator{
	typedef enum TOp{
		unknown,//!<Inconnu
		ne,//!<Diffrent
		eq,//!<Egal
		lt,//!<Strictement infrieur
		gt,//!<Strictement suprieur
		le,//!<Infrieur ou gal
		ge//Suprieur ou gal
	}TOp;
	static char const*opToStr(TOp op);
	static TOp strToOp(char const*str);
	static bool compare(float val1,TOp op,float val2);
	static bool compare(double val1,TOp op,double val2);
};

struct ICondFctFloat{//Interface des fonctors condition avec argument rel (ce n'est jamais un objet python)
	virtual ~ICondFctFloat(){}
	virtual bool operator()(float val)const{return true;}
};

struct CCondFctFloat:ICondFctFloat{//Fonctor condition construit  partir d'une fonction
	bool (*m_Condition)(float);
	CCondFctFloat(bool (*condition)(float)):m_Condition(condition){}
	inline bool operator()(float val)const{
		return m_Condition(val);
	}
};

class CCondCompFloat:public ICondFctFloat{//Fonctor condition construit  partir d'un oprateur de comparaison et d'une valeur
	float m_Val;
	TOperator::TOp m_Op;
public:
	CCondCompFloat():m_Val(0),m_Op(TOperator::eq){}
	CCondCompFloat(char const * op, float value):m_Val(value) {
		setOp(op);
	}
	bool operator()(float val)const override;
	bool setOp(char const*op);
	void setValue(float val){m_Val=val;}
	TOperator::TOp op()const{return m_Op;}
	float value()const{return m_Val;}
	char const*opStr()const;
};

/**Interface d'accs  une branche de mmorisation de squence.
La branche contient la date de mmorisation, l'indice de branche  la date et les valeurs mmorises pour chaque lment monitor  cet occasion.
*/
class PYC_PUBLIC IBranch{
protected:
	IBranch(){}
	virtual~IBranch(){}
public:
	virtual PyTime time()const=0;//!<Retourne l'instant du pas de temps courant
	virtual unsigned index()const=0;//!<Retourne l'indice de branche pour le pas de temps courant
	virtual float value(IMonitored const*obj,bool*exists=NULL)const=0;//!<Retourne la valeur de l'lment obj  l'instant courant s'il existe
	virtual std::vector<IMonitored const*>monitoredElts()const=0;//!<Retourne la liste des lments monitors  l'instant courant
};

/**Interface d'accs  l'itrateur de squence*/
class PYC_PUBLIC ISeqIter{
public:
	virtual~ISeqIter(){}
	virtual void restart()=0;//!<Replace l'itrateur en dbut de squence
	virtual bool atEnd()const=0;//!<Retourne vrai si l'itrateur a atteint la fin de la squence
	virtual bool atBegin()const=0;//!<Retourne vrai si l'itrateur est en dbut de la squence
	virtual void next()=0;//!<Avance au pas d'enregistrement suivant (le temps peut ne pas changer)
	virtual void previous()=0;//!<Recule au pas d'enregistrement prcdent (le temps peut ne pas changer)
	virtual IBranch const*branch()const=0;//!<Retourne la branche courante
	virtual PyTime time()const{return branch()->time();}//!<Retourne l'instant du pas de temps courant
	virtual unsigned index()const{return branch()->index();}//!<Retourne l'indice de branche pour le pas de temps courant
	virtual float value(IMonitored const*obj,bool*exists=NULL)const=0;//!<Retourne la valeur de l'lment obj  l'instant courant s'il existe
	virtual std::vector<IMonitored const*>monitoredElts()const{return !atEnd()?branch()->monitoredElts():std::vector<IMonitored const*>();}//!<Retourne la liste des lments monitors  l'instant courant
};

/**Interface d'accs  une squence.
La squence permet d'accder aux valeurs des lments monitors  n'importe quel instant.
Pour suivre le droulement d'une squence il faut utiliser un itrateur obtenu avec la mthode iterator().
*/
class PYC_PUBLIC ISequence{
protected:
	ISequence(){}
	virtual~ISequence(){}
public:
	/**Retourne un itrateur sur la squence.*/
	virtual ISeqIter*iterator()const=0;
	/**Retourne le numro de la portion de squence*/
	virtual SEQ_ID num()const=0;
	/**Retourne la vraisemblance de la squence.*/
	virtual double likelyhood()const=0;
	/**Modifie l'indice de conservation de la squence.*/
	virtual void setToKeep(bool toKeep)=0;
	/**Retourne vrai si la squence doit tre conserve.*/
	virtual bool toKeep()const=0;
	/**Ajoute une prolongation  la squence*/
	virtual void addExtend(ITransition*tr,PyTime date)=0;
	/**Retourne la valeur de l'objet obj  l'instant t.*/
	virtual float value(IMonitored const*obj,PyTime t)const=0;
	/**Calcule les valeurs extrmes de l'lment monitor (min et max doivent tre initialiss).*/
	virtual void extremeValues(IMonitored const*obj,float&min,float&max)const=0;
	std::vector<float>extremeValues(IMonitored const*obj)const{std::vector<float>v(2,(float)HUGE_VAL);extremeValues(obj,v[0]=(float)-HUGE_VAL,v[1]);return v;}
	/**Calcule les temps de sjour entre les bornes.*/
	virtual void residenceTimes(IMonitored const*obj,PyTime it,PyTime ft,std::vector<float>const&lims,std::vector<PyTime>&resTs)const=0;
	std::vector<PyTime>residenceTimes(IMonitored const*obj,PyTime it,PyTime ft,std::vector<float>const&lims)const{std::vector<PyTime>v;residenceTimes(obj,it,ft,lims,v);return v;}
	/**Retourne le temps de sjour satisfaisant la condition.*/
	virtual float residenceTime(IMonitored const*obj,PyTime it,PyTime ft,ICondFctFloat const&condition)const=0;
	virtual float residenceTime(IMonitored const*obj,PyTime it,PyTime ft,char const*op,float value)const{
		return residenceTime(obj,it,ft,CCondCompFloat(op,value));
	}
	virtual float residenceTime(IMonitored const*obj,PyTime it,PyTime ft,bool (*condition)(float))const{
		return residenceTime(obj,it,ft,CCondFctFloat(condition));
	}
	/**Retourne la date de premire ralisation de la condition*/
	virtual PyTime realized(IMonitored const*obj,ICondFctFloat const&condition)const=0;
	virtual PyTime realized(IMonitored const*obj,char const*op,float value)const{
		return realized(obj,CCondCompFloat(op,value));
	}
	PyTime realized(IMonitored const*obj,bool (*condition)(float))const{
		return realized(obj,CCondFctFloat(condition));
	}
	/**Retourne vrai dans real[i] si la condition est satisfaite avant ts[i].*/
	virtual void realized(IMonitored const*obj,ICondFctFloat const&condition,std::vector<PyTime>const&ts,std::vector<bool>&real)const=0;
	void realized(IMonitored const*obj,char const*op,float value,std::vector<PyTime>const&ts,std::vector<bool>&real)const{
		realized(obj,CCondCompFloat(op,value),ts,real);
	}
	void realized(IMonitored const*obj,bool (*condition)(float),std::vector<PyTime>const&ts,std::vector<bool>&real)const{
		realized(obj,CCondFctFloat(condition),ts,real);
	}
	std::vector<bool>realized(IMonitored const*obj,ICondFctFloat const&condition,std::vector<PyTime>const&ts)const{std::vector<bool>v;realized(obj,condition,ts,v);return v;}
	std::vector<bool>realized(IMonitored const*obj,char const*op,float value,std::vector<PyTime>const&ts)const{
		return realized(obj,CCondCompFloat(op,value),ts);
	}
	std::vector<bool>realized(IMonitored const*obj,bool (*condition)(float),std::vector<PyTime>const&ts)const{
		return realized(obj,CCondFctFloat(condition),ts);
	}
	/**Retourne le nombre de branches de la squence*/
	virtual unsigned branchCount()const=0;
	/**Retourne la ime branche de la squence*/
	virtual IBranch const*branch(unsigned i)const=0;
	/**Retourne la date d'arrt de la squence*/
	virtual PyTime endTime()const=0;
	/**Retourne la cause d'arrt de la squence*/
	virtual char const*endCause()const=0;
	/**Retourne l'intgrale de la valeur de l'objet obj entre les temps it et ft.*/
	virtual float integralValue(IMonitored const*obj,PyTime it,PyTime ft)const=0;
	/**Retourne dans values les valeurs de l'objet obj aux instants ts.*/
	virtual void values(IMonitored const*obj,std::vector<PyTime>const&ts,std::vector<float>&values)const=0;
	std::vector<float>values(IMonitored const*obj,std::vector<PyTime>const&ts)const{std::vector<float>v;values(obj,ts,v);return v;}
};

/**Interface des filtres de squence (ce n'est jamais un objet python)
*/
class PYC_PUBLIC IFilterFct{
	friend class CSystem;
	friend class CSystemP;
	friend class CAnalyser;
public:
	virtual ~IFilterFct(){}
	virtual bool operator()(ISequence const&seq)const=0;
	static IFilterFct*newPatternFilter(char const*pattern);
	static IFilterFct*newConditionFilter(char const*object,PyTime time,char const*op,float val);
};

struct TIndicatorType{
	/**Enumr de dfinition du mode de mmorisation des indicateurs.
		Les lments du type peuvent tre cumuls.
	*/
	typedef enum EIndicatorType{
		no_value=0,
		mean_values=1,
		std_dev=2,
		all_values=4,
		quantile_le   = 8,
		quantile_gt   = 16,
		distribution=32,
		monitor_values=512
	}EIndicatorType;
	static char const*st_Names[8];
};
struct TComputationType{
	/**Enumr de dfinition de la valeur calcule par l'indicateur.*/
	typedef enum EComputationType{
		simple=0,
		res_time,
		residence_time=res_time,
		nb_visits,
		realized
	}EComputationType;
	static char const*st_Names[6];
};
/**Interface d'acces aux indicateurs.
*/
class PYC_PUBLIC IIndicator:public IMonitored,protected CMethod{
protected:
	IIndicator(){}
	IIndicator(CNamedComp*comp,char const*name,float(CNamedComp::*fct)()):CMethod(comp,name,fct){}
	virtual ~IIndicator(){}
public:
	char const*basename()const override{return CMethod::basename();}
	char const*name()const override{return CMethod::basename();}
	virtual void setRestitutions(int type)=0;//!<Modifie les restitutions de l'indicateur
	virtual int restitutions()const=0;//!<Retourne les restitutions de l'indicateur
	virtual void setComputation(TComputationType::EComputationType comp)=0;//!<Modifie la valeur calcule par l'indicateur
	virtual TComputationType::EComputationType computation()const=0;//!<Retourne la valeur calcule par l'indicateur
	virtual void setPctQuantileLeValue(float fracMin)=0;//!<Modifie le pourcentage de calcul du fractile min
	virtual void setPctQuantileGtValue(float fracMax)=0;//!<Modifie le pourcentage de calcul du fractile max
	virtual float pctQuantileLeValue()const=0;//!<Retourne le pourcentage de calcul du fractile min
	virtual float pctQuantileGtValue()const=0;//!<Retourne le pourcentage de calcul du fractile max
	virtual void setDistribLimits(std::vector<float>lims)=0;//!<Modifie les limites de calcul de la distribution
	virtual std::vector<float>distribLimits()const=0;//!<Retourne les limites de calcul de la distribution
	virtual std::vector<float>const&values(int i)const=0;//!<Retourne toutes les ime valeurs
	virtual float mean(int i)const=0;//!<Retourne la ime moyenne
	virtual float stdDev(int i)const=0;//!<Retourne le ime cart type
	virtual float quantileLe(int i)const=0;//!<Retourne le ime fractile min
	virtual float quantileGt(int i)const=0;//!<Retourne le ime fractile max
	virtual float confInt(float pct,int i)const=0;//!<Retourne le ime intervalle de confiance  pct%
	virtual std::vector<float>distribution(int i)const=0;//!<Retourne les probabilits de la ime distribution
	virtual std::vector<float>meanDistribution()const=0;//!<Retourne les probabilits de la distribution moyenne sur tous les instants
	virtual std::vector<float>means()const=0;//!<retourne les moyennes aux diffrents instants
	virtual std::vector<float>stdDevs()const=0;//!<retourne les carts types aux diffrents instants
	virtual std::vector<float>quantilesLe()const=0;//!<retourne les fractiles min aux diffrents instants
	virtual std::vector<float>quantilesGt()const=0;//!<retourne les fractiles min aux diffrents instants
	virtual std::vector<float>confInts(float pct)const=0;//!<retourne les intervalles de confiance  pct% aux diffrents instants
	virtual void setToKeep(bool toKeep)=0;//!<Modifie l'indice de conservation de l'indicateur
	virtual bool toKeep()const=0;//!<Retourne vrai si l'indicateur doit tre conserv
};

class ITarget:protected CMethod{
protected:
	ITarget(){}
	ITarget(CNamedComp*comp,char const*name,bool(CNamedComp::*fct)()):CMethod(comp,name,fct){}
	ITarget(CNamedComp*comp,char const*name,PyObject*pVoid):CMethod(comp,name,pVoid){}
	virtual ~ITarget(){}
public:
	char const*basename()const{return CMethod::basename();}
	char const*name()const{return CMethod::basename();}
	virtual void setInhibited(bool value)=0;
	virtual bool inhibited()const=0;
};

/**Interface commune  l'ensemble des variables et des rfrences.
*/
class PYC_PUBLIC IVarBase:public CNamedComp{
	friend class CSystemState;
	virtual void addSensitiveMethod(SSensitiveMethod const&method)=0;
	CNamedComp*p()const;//!<Juste pour accder au parent pour construire la sensitive method
protected:
	CNX_ID m_N;//!<Nombre maximum de liens accepts par la variable.
	IVarBase(char const*name,CNamedComp*parent):CNamedComp(name,parent),m_N(0xffffffff){}
	~IVarBase(){}
public:
	/**Retourne le type de la valeur courante de la variable.*/
	virtual std::type_info const&typeInfo()const{return typeid(bool);}
	/**Retourne le nombre maximum de connexion sur la variable.*/
	virtual CNX_ID cnctMax()const{return m_N;}
	/**Ajoute une mthode sensible aux modifications de la variable.
	@param name nom de la mthode
	@param method mthode sensible  ajouter
	@param sens sens de sensibilit de la mthode (0 tous, 1 accroissement, -1 rduction)
	*/
	template<typename Type>void addSensitiveMethod(char const*name,void(Type::*method)(),int sens=0){addSensitiveMethod(SSensitiveMethod(p(),name,static_cast<void(CNamedComp::*)()>(method),sens));}
	void addPySensitiveMethod(char const*name,PyObject*pVoid,int sens){addSensitiveMethod(SSensitiveMethod(p(),name,pVoid,sens));}
	/**Supprime la mthode de nom name des mthodes sensible aux modifications de la variable*/
	virtual void removeSensitiveMethod(char const*name)=0;
	virtual void clearSensitiveMethod()=0;
	/**Modifie le nombre maximum de connexion sur la variable*/
	void setCnctMax(CNX_ID N){m_N=N;}
};

/**Interface de gestion des variables.
	Les variables peuvent tre des variables d'tat ou des automates ou des tats.
*/
class PYC_PUBLIC IVariable : public IVarBase,public IMonitored{
	void value(bool&res)const{res=bValue();}
	void value(int&res)const{res=iValue();}
	void value(float&res)const{res=fValue();}
	void value(double&res)const{res=dValue();}
	void value(std::complex<double>&res)const{res=cValue();}
	void value(std::string&res)const{res=sValue();}
protected:
	IVariable():IVarBase(NULL,NULL){}
	IVariable(CNamedComp&parent,char const*name):IVarBase(name,&parent){}
	~IVariable(){}
public:
	char const*basename()const override{return CNamedComp::basename();}
	char const*name()const override{return CNamedComp::name();}
	CNamedComp*parent()const override{return CNamedComp::parent();}
	virtual CVariable const*variable()const{return NULL;}
	/**Modifie la rinitialisation de la variable*/
	virtual void setReinitialized(bool toReinit){ILogManager::glLogManager().throwError("Mthode inutilisable sur cette classe");}
	/**Retourne la rinitialisation de la variable*/
	virtual bool isReinitialized()const{return false;}
	/**Modifie la gestion de l'interpolation des valeurs de la variable*/
	virtual void setInterpolated(bool interp);
	/**Retourne vrai si les valeurs de la variable doivent tre interpoles*/
	virtual bool isInterpolated()const{return false;}
	/**Modifie la drive de la valeur pour les ODE*/
	virtual void setDvdtODE(double dvdt)=0;
	/**Modifie le fait que la variable soit modifiable de l'extrieur*/
	virtual void setModifiable(bool res)=0;
	/**Retourne la drive de la valeur de la variable (uniquement pour les variables double)*/
	virtual double derivative(double dt)const=0;
	/**Retourne le pas de calcul de la drive*/
	virtual double dtDerivative(double dt)const{return derivative(dt);/*Pour obtenir le message d'erreur par dfaut*/}
	/**Dsigne la variable dont la variable courante est l'intgrale.*/
	virtual void integrate(IVariable*var);
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de complexe.*/
	virtual void setValue(std::complex<double>val)=0;
	inline void setCValue(std::complex<double>val){setValue(val);}
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de double.*/
	virtual void setValue(double val)=0;
	inline void setDValue(double val){setValue(val);}
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de float.*/
	virtual void setValue(float val)=0;
	inline void setFValue(float val){setValue(val);}
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de int.*/
	virtual void setValue(int val)=0;
	inline void setIValue(int val){setValue(val);}
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de bool.*/
	virtual void setValue(bool val)=0;
	inline void setBValue(bool val){setValue(val);}
	/**Modifie la valeur courante (ou la valeur initiale si la simulation n'est pas lance) sous forme de string.*/
	virtual void setValue(char const*val)=0;
	inline void setSValue(char const*val){setValue(val);}
	/**Retourne la valeur courante sous forme de complexe.*/
	virtual std::complex<double> cValue()const=0;
	/**Retourne la valeur courante sous forme de double.*/
	virtual double dValue()const=0;
	/**Retourne la valeur courante sous forme de float.*/
	virtual float fValue()const=0;
	/**Retourne la valeur courante sous forme de int.*/
	virtual int iValue()const=0;
	/**Retourne la valeur courante sous forme de bool.*/
	virtual bool bValue()const=0;
	/**Retourne la valeur courante sous forme de string.*/
	virtual std::string sValue()const=0;
	/**Retourne la valeur courante.*/
	template<typename T>T value()const{
		T res;
		value(res);
		return res;
	}
	/**Retourne la valeur initiale sous forme de complexe.*/
	virtual std::complex<double> cInitValue()const=0;
	/**Retourne la valeur initiale sous forme de double.*/
	virtual double dInitValue()const=0;
	/**Retourne la valeur initiale sous forme de float.*/
	virtual float fInitValue()const=0;
	/**Retourne la valeur initiale sous forme de int.*/
	virtual int iInitValue()const=0;
	/**Retourne la valeur initiale sous forme de bool.*/
	virtual bool bInitValue()const=0;
	/**Retourne la valeur initiale sous forme de string.*/
	virtual std::string sInitValue()const=0;
	/**Retourne 0 si la valeur courante est indentique  la valeur initiale et + ou -1 suivant le sens de variation.*/
	virtual int valueChanged()const=0;
	/**Applique la valeur de redmarrage  la valeur courante.*/
	virtual void reset()=0;
	/**Demande la gestion du monitoring de la variable.*/
	virtual void monitorValue()const=0;
};

/**Interface de gestion des rfrences  des variables.
	Les rfrences sont le reflet de variables situes dans d'autres composants.
*/
class PYC_PUBLIC IReference : public IVarBase{
	void value(bool&res,CNX_ID i)const{res=bValue(i);}
	void value(int&res,CNX_ID i)const{res=iValue(i);}
	void value(float&res,CNX_ID i)const{res=fValue(i);}
	void value(double&res,CNX_ID i)const{res=dValue(i);}
	void value(std::complex<double>&res,CNX_ID i)const{res=cValue(i);}
	void value(std::string&res,CNX_ID i)const{res=sValue(i);}
	using IVarBase::typeInfo;
protected:
	IReference(CNamedComp&parent,char const*name):IVarBase(name,&parent){}
public:
	/**Retourne le type de la ime connexion*/
	virtual std::type_info const&typeInfo(CNX_ID i)const=0;
	/**Retourne le nombre de valeurs connues par la rfrence (connexions relles ou valeurs fixes).*/
	virtual CNX_ID cnctCount()const=0;
	virtual CNX_ID nbCnx()const{return cnctCount();}
	/**Retourne la valeur courante de la ime connexion sous forme de complexe.*/
	virtual std::complex<double>cValue(CNX_ID i)const=0;
	/**Retourne la valeur courante de la ime connexion sous forme de double.*/
	virtual double dValue(CNX_ID i)const=0;
	/**Retourne la valeur courante de la ime connexion sous forme de float.*/
	virtual float fValue(CNX_ID i)const=0;
	/**Retourne la valeur courante de la ime connexion sous forme de int.*/
	virtual int iValue(CNX_ID i)const=0;
	/**Retourne la valeur courante de la ime connexion sous forme de bool.*/
	virtual bool bValue(CNX_ID i)const=0;
	/**Retourne la valeur courante de la ime connexion caste au type T.*/
	template<typename T>T value(CNX_ID i)const{
		T res;
		value(res,i);
		return res;
	}
	/**Retourne la valeur courante de la ime connexion sous forme de string.*/
	virtual std::string sValue(CNX_ID i)const=0;
	/**Retourne les valeurs courantes des connexions sous forme de complexe.*/
	virtual CNX_ID cValues(std::complex<double>*tab)const=0;
	/**Retourne les valeurs courantes des connexions sous forme de double.*/
	virtual CNX_ID dValues(double*tab)const=0;
	/**Retourne les valeurs courantes des connexions sous forme de float.*/
	virtual CNX_ID fValues(float*tab)const=0;
	/**Retourne les valeurs courantes des connexions sous forme d'entiers.*/
	virtual CNX_ID iValues(int*tab)const=0;
	/**Retourne les valeurs courantes des connexions sous forme de boolens.*/
	virtual CNX_ID bValues(bool*tab)const=0;
	/**Retourne la rfrence associe  la ime connexion.*/
	virtual IReference*reference(CNX_ID i)const=0;
	/**Retourne la variable associe  la ime connexion.*/
	virtual IVariable*variable(CNX_ID i)const=0;
	/**Retourne la IVarBase associe  la ime connexion.*/
	virtual IVarBase*varBase(CNX_ID i)const=0;
	/**Prend une quantit res de ressource  la ime variable connecte.*/
	virtual void takeResource(CNX_ID i,double res)=0;
	/**Restitue une quantit res de ressource  la ime variable connecte.*/
	virtual void restoreResource(CNX_ID i,double res)=0;
	/**Modifie la valeur de la ime variable connecte.*/
	virtual void setValue(CNX_ID i,double val)=0;
	/**Modifie la drive de la ime variable connecte (uniquement si c'est une variable double).*/
	virtual void setDvdt(CNX_ID i,double val)=0;
	/**Lie la rfrence  une autre rfrence ou variable vb*/
	virtual void bind(IVarBase&vb)=0;
	/**Dlie la rfrence de la rfrence ou de la variable vb*/
	virtual void unbind(IVarBase&vb)=0;
	/**Dlie la ime connexion*/
	virtual void unbind(CNX_ID i)=0;
	/**Calcule un ou sur l'ensemble des connexions (def est la valeur en l'absence de connexion)*/
	virtual bool orValue(bool def=false)const=0;
	/**Calcule un et sur l'ensemble des connexions (def est la valeur en l'absence de connexion)*/
	virtual bool andValue(bool def=true)const=0;
	/**Calcule une somme de l'ensemble des connexions (def est la valeur en l'absence de connexion)*/
	virtual double sumValue(double def=0)const=0;
	/**Calcule un produit l'ensemble des connexions (def est la valeur en l'absence de connexion)*/
	virtual double productValue(double def=1)const=0;
	/**Remplit le vecteur avec les indices qui vrifient la condition*/
	virtual int valuesThat(ICondFctFloat const&cf, std::vector<int> & indexes)const=0;
	virtual int valuesThat(bool(condition)(float), std::vector<int> & indexes)const{return valuesThat(CCondFctFloat(condition),indexes);}
};

/**Interface de gestion des botes de messages.
*/
class PYC_PUBLIC IMessageBox:public CNamedComp{
protected:
	IMessageBox(CNamedComp&parent,char const*name):CNamedComp(name,&parent){}
public:
	/**Ajoute une variable  connecter.*/
	virtual void addExport(IVarBase*p,char const*aliasName=NULL)=0;
	/**Retourne la variable exporte sous l'alias aliasName (exception si c'est une rfrence).*/
	virtual IVariable*exportedVariable(char const*aliasName)const=0;
	/**Ajoute une rfrence  connecter et dfinit le nom de la variable  lui associer.*/
	virtual void addImport(IReference*p,char const*varName=NULL)=0;
	/**Ajoute une rfrence  connecter facultativement et dfinit le nom de la variable  lui associer et sa valeur par dfaut s'il n'est pas connect.*/
	virtual void addOptionalImport(IReference*p,int type, double defaultValue, char const*alias)=0;
	//*Retourne vrai si la connection peut tre effectue*/
	virtual bool canConnectTo(IMessageBox&other)const=0;
	//*Retourne vrai si la connection existe dj*/
	virtual bool isConnectedTo(IMessageBox&other)const=0;
	/**Connecte la bote de messages  une autre bote de messages.*/
	virtual void connectTo(IMessageBox&other,double weight)=0;
	void connectTo(IMessageBox&other){connectTo(other,0);}//Pour le python
	/**deconnecte la bote de messages d'une autre bote de messages.*/
	virtual void disconnectFrom(IMessageBox&other)=0;
	/**deconnecte la bote de messages des autres botes de messages.*/
	virtual void disconnect()=0;
	/**Retourne le nombre de connexions*/
	virtual CNX_ID cnctCount()const=0;
	/**Retourne le poids de la ime connection*/
	virtual double weight(CNX_ID i)const=0;
	/**Retourne le ime composant connect*/
	virtual CComponent*component(CNX_ID i)const=0;
	/**Retourne la ime bote de message connecte*/
	virtual IMessageBox*cnct(CNX_ID i)const=0;
	/**Retourne sous forme de complexe la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual std::complex<double> cValue(CNX_ID i,char const*aliasName)const=0;
	/**Retourne sous forme de double la valeur la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual double dValue(CNX_ID i,char const*aliasName)const=0;
	/**Retourne sous forme de float la valeur la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual float fValue(CNX_ID i,char const*aliasName)const=0;
	/**Retourne sous forme de int la valeur la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual int iValue(CNX_ID i,char const*aliasName)const=0;
	/**Retourne sous forme de bool la valeur la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual bool bValue(CNX_ID i,char const*aliasName)const=0;
	/**Retourne sous forme de string la valeur la valeur de la ime variable connecte exporte sous l'alias aliasName.*/
	virtual std::string sValue(CNX_ID i,char const*aliasName)const=0;
};

struct SBindingMM;
struct SBindingM;
struct SBinding;
/**Modificateur de loi de probabilit
Le foncteur modifie la valeur "normale" qui est fournie en iniVal en la multipliant par le coefficient m_CoefModif.
*/
struct SDistLawModifier{
	double m_CoefModif;
	virtual double operator()(double iniVal)const{return iniVal*m_CoefModif;}
};
struct TLawType{
	/**Dfinit les types de loi prdfinis dans le coeur de Pycatshoo*/
	typedef enum ELawType{
		unknown=-1,//!<Loi certaine
		inst,//!<Loi instantane
		expo,//!<Loi exponentielle
		cstt,//!<Loi temps constant
		defer=cstt,
		cstd,//!<Loi date constante
        pdefer//same as defer with a probability //HC
	}ELawType;
};
/**Interface de gestion des lois de probabilit.
*/
class PYC_PUBLIC IDistLaw{
	friend class CDistLawInst;
	friend class ITransition;
	friend class CTransition;
	LAWP_ID m_NbParam;
	CNamedComp&m_Parent;
	SBindingMM*m_Parameters;//!<Dfinition premire des paramtres permettant de mettre  jour les valeurs numriques
	double*m_ParamValues;//!<Valeurs des paramtres utilises pour le calcul du temps d'attente
	void setParameter(double(CNamedComp::*fct)(),LAWP_ID place);
	void reset();//!<Replace les valeurs initiales
	void insertAt(LAWP_ID place,bool bAdd=false);//!<Cre une place en place
	void incRef();
	unsigned int m_NbCppRef;//!<Nombre de rfrences d'un objet C++
	PyObject*m_Self;//!<Rfrence  l'objet python correspondant (NULL, si c'est un objet purement C++).
	IDistLaw(IDistLaw const&);//Interdit
	IDistLaw&operator=(IDistLaw const&);//Interdit
protected:
	virtual~IDistLaw(void);
	IDistLaw(PyObject*self,CNamedComp&parent);
public:
	explicit IDistLaw(CNamedComp&parent);
	/**Donne le nom du type de distribution*/
	virtual char const*name()const{return NULL;}
	/**Retourne le type de loi (parmi TLawType pour les type standard)*/
	virtual int type()const{return TLawType::unknown;}
	void deleteDLaw();//!<Detruit la loi sauf si m_Self est non null
	/**Dfinit le nombre de paramtres de la loi (par dfaut 1)*/
	void setNbParameters(LAWP_ID nb){if(nb>m_NbParam)insertAt(nb-1);else m_NbParam=nb;}
	/**Insert un paramtre sous forme de valeur.*/
	void insertParameter(double value,LAWP_ID place=0);
	/**Insert un paramtre sous forme de variable.*/
	void insertParameter(IVariable&var,LAWP_ID place=0);
	/**Dfinit un paramtre par connexion d'une variable.*/
	void setParameter(IVariable&var,LAWP_ID place=0);
	/**Dfinit un paramtre par une mthode*/
	template<typename Type>void setParameter(double (Type::*fct)(),LAWP_ID place=0){setParameter(static_cast<double (CNamedComp::*)()>(fct),place);}
	/**Modifie la valeur d'un paramtre.*/
	void setParameter(double value,LAWP_ID place=0);
	/**Dfinit un paramtre par une fonction Python*/
	void setPyParameter(char const*name,PyObject*pyObj,LAWP_ID place);
	/**Dfinit un coefficient de modification pour un paramtre existant*/
	void setParameterModifier(double coef,LAWP_ID place);
	/**Retourne le modificateur du ime paramtre*/
	double parameterModifier(LAWP_ID place)const;
	/**Retourne la valeur d'un paramtre*/
	virtual double parameter(LAWP_ID place=0)const{return m_ParamValues[place];}
	/**Retourne le temps d'attente tir alatoirement avec les paramtres courants de telle sorte que le dlai soit suprieur  dMin*/
	virtual PyTime delayBeyond(PyTime dMin)const{return dMin;}
	/**Retourne le temps d'attente tir alatoirement avec les paramtres courants*/
	virtual PyTime delay()const{return delayBeyond(0.);}
	/**Retourne le temps minimal d'attente*/
	virtual PyTime delayMin()const{return 0.;}
	/**Retourne le temps minimal d'attente*/
	virtual PyTime delayMax()const{return HUGE_VAL;}
	/**Retourne la valeur de la densit de probabilit*/
	virtual double density(PyTime t)const{return 0.;}
	/**Retourne la valeur de la fonction de rpartition*/
	virtual double distribution(PyTime t)const{return 0.;}
	/**Retourne l'indexe de la transition de sortie tir alatoirement avec les paramtres courants ( partir de 0 jusqu' nbIndex-1)*/
	virtual TGT_ID index()const{return 0;}
	/**Retourne le nombre de paramtres de la loi*/
	inline LAWP_ID nbParam()const{return m_NbParam;}
	/**Retourne le nombre de sorties gres par la loi.*/
	virtual TGT_ID nbIndex()const{return 1;}
	/**Retourne faux si les paramtres ont chang*/
	bool upToDate();
	/**Construit une des lois prdfinies*/
	static IDistLaw*newLaw(CNamedComp&parent,TLawType::ELawType type,IVariable&var);
	/**Construit une des lois prdfinies*/
	static IDistLaw*newLaw(CNamedComp&parent,TLawType::ELawType type,double param);
	/**Construit une des lois prdfinies*/
	template<typename Type>static IDistLaw*newLaw(CNamedComp&parent,TLawType::ELawType type,double (Type::*fct)()){IDistLaw*dl=newLaw(parent,type,0);dl->setParameter(fct);return dl;}
	static IDistLaw*newPyLaw(CNamedComp&parent,TLawType::ELawType type,char const*name,PyObject*pVoid){IDistLaw*dl=newLaw(parent,type,0);dl->setPyParameter(name,pVoid,0);return dl;}
	static IDistLaw*newPyLaw(CNamedComp&parent,TLawType::ELawType type,char const*name){return newPyLaw(parent,type,name,NULL);}
};

struct TModificationMode{
	/**Enumr de dfinition de l'aspect modifiable de la transition.*/
	typedef enum EModificationMode{
		not_modifiable,//!<Transition non modifiable
		not_modif=not_modifiable,
		discrete_modification,//!<Transition modifiable  chaque pas
		modif=discrete_modification,
		modifiable=modif,
		continuous_modification,//!<Transition modifiable continment (dans les ODE)
		cont_modif=continuous_modification,
		cont_modifiable=continuous_modification
	}EModificationMode;
};
struct TTransType{
	/**Enumr de dfinition de type de franchissement de la transition.*/
	typedef enum ETransType{
		trans=1,//!<Transition standard
		fault=2,//!<Dfaillance
		rep=3//!<Rparation
	}ETransType;
	static char const*st_Names[];
};
/**Interface de gestion des transitions d'un tat  un autre.
	Une transition permet de contrler le passage d'un tat  un autre.
	Une transition gre :
	- une condition par l'intermdiaire d'une rfrence m_Condition,
	- des paramtres de loi par l'intermdiaire de la rfrence m_Parameters.
	Une transition connat son tat de dpart et ses tats d'arrive potentiels.
	Il ne peut y avoir plusieurs tats d'arrive que pour une transition INS.

	Les modles actuellement reconnus sont :
	- INS : les paramtres sont les proba de passage dans les tats cibles (le dernier paramtre peut tre ignor)
	- EXP : l'unique paramtre est le taux de passage par unit de temps (seconde)
	- TC  : l'unique paramtre est la dure d'attente en seconde

	La transition peut tre qualife de :
	- trans : transition standard,
	- def : dfaillance,
	- rep : rparation.

	La transition peut tre :
	- interruptible (si la condition est invalide pendant l'attente, l'attente est interrompue),
	- modifiable (si les paramtres sont modifis pendant l'attente, celle-ci est recalcule).
*/
class PYC_PUBLIC ITransition : public CNamedComp,public IMonitored{
protected:
	SBindingM&m_Condition;//!<Dfinition de la condition de franchissement de la transition
	bool m_NCond;//!<Si vrai, la condition est utilise  l'envers
	bool m_NCondInit;//!<Si vrai, la condition est initialement utilise  l'envers
	bool m_Interruptible;//!<Si vrai, la transition peut tre interrompue si la condition n'est plus valide (par dfaut faux)
	TModificationMode::EModificationMode m_Modifiable;//!<Si 1 la transition peut tre modifie aprs son lancement, si 2 elle est continuement modifiable (intgration) (par dfaut non modifiable)
	IDistLaw*m_DistLaw;//!<Loi de distribution des probabilits
	virtual void addSensitiveMethod(SSensitiveMethod const&method)=0;
	void setCondition(bool(CNamedComp::*fct)(),bool negate);
	ITransition(CNamedComp&parent,char const*name);
	virtual~ITransition();
public:
	char const*basename()const override{return CNamedComp::basename();}
	char const*name()const override{return CNamedComp::name();}
	CNamedComp*parent()const override{return CNamedComp::parent();}
	/**Retourne la loi de probabilit de la transition*/
	IDistLaw*distLaw()const{return m_DistLaw;}
	/**Modifie la loi de probabilit de la transition*/
	void setDistLaw(IDistLaw*distLaw);
	/**Modifie la loi de probabilit de la transition*/
	IDistLaw*setDistLaw(TLawType::ELawType type,double param){setDistLaw(IDistLaw::newLaw(*parent(),type,param));return distLaw();}
	IDistLaw*setDistLaw(TLawType::ELawType type,IVariable&param){setDistLaw(IDistLaw::newLaw(*parent(),type,param));return distLaw();}
	template<typename Type>IDistLaw*setDistLaw(TLawType::ELawType type,double(Type::*fct)()){setDistLaw(IDistLaw::newLaw(*parent(),type,fct));return distLaw();}
	IDistLaw*setPyDistLaw(TLawType::ELawType type,char const*name,PyObject*pVoid){setDistLaw(IDistLaw::newPyLaw(*parent(),type,name,pVoid));return distLaw();}
	IDistLaw*setPyDistLaw(TLawType::ELawType type,char const*name){setPyDistLaw(type,name,NULL);return distLaw();}
	/**Modifie le caractre interruptible de la transition (par dfaut faux).*/
	void setInterruptible(bool interruptible){m_Interruptible=interruptible;}
	/**Retourne le cratre interruptible de la transition.*/
	bool interruptible()const{return m_Interruptible;}
	/**Modifie le caractre modifiable de la transition (par dfaut faux).*/
	void setModifiable(TModificationMode::EModificationMode modifiable){m_Modifiable=modifiable;}
	/**Retourne le cratre modifiable de la transition.*/
	TModificationMode::EModificationMode modifiable()const{return m_Modifiable;}
	/**Retourne l'tat de dpart de la transition*/
	virtual IState const*startState()const=0;
	/**Retourne le nombre de cibles de la transition*/
	virtual TGT_ID targetCount()const=0;
	/**Insert un tat cible  la transition et dfinit le type de la transition vers cet tat.*/
	virtual void insertTarget(IState*end,int type,int place)=0;
	/**Ajoute un tat cible  la transition et dfinit le type de la transition vers cet tat.*/
	virtual void addTarget(IState*end,int type=TTransType::trans)=0;
	/**Retourne l'tat cible d'indice i.*/
	virtual IState*getTarget(TGT_ID i)const=0;
	/**Retourne le type de franchissement effectu pour atteindre le ime tat cible.*/
	virtual int getTargetType(TGT_ID ind)const=0;
	/**Inhibe ou dsinhibe les sorties vers l'tat*/
	virtual void inhibateTarget(IState*st,bool inhibate)=0;
	/**Inhibe ou dsinhibe la sortie vers les cibles de type type*/
	virtual void inhibateTarget(int type,bool inhibate)=0;
	/**Renseigne la condition par connexion  une variable ou  un tat.*/
	void setCondition(IVariable&var,bool negate=false);
	void setCondition(IVariable*var,bool negate=false){setCondition(*var,negate);}
	/**Renseigne la condition par une fonction boolenne.*/
	template<typename FctType> void setCondition(FctType fct,bool negate=false){setCondition(static_cast<bool(CNamedComp::*)()>(fct),negate);}
	/**Renseigne la condition par une fonction Python dfinie soit par son nom, soit par son objet.*/
	void setPyCondition(char const*name,PyObject*pyObj,bool negate=false);
	/**Renseigne la condition par une valeur boolenne.*/
	void setCondition(bool val);
	/**Retourne la valeur actuelle de la condition.*/
	bool getCondition()const;
	/**Retourne la date de franchissement prvue (ou -1 s'il n'y a pas de date prvue).*/
	virtual PyTime endTime()const=0;
	/**Retourne la date de dcision d'activation*/
	virtual PyTime startTime()const=0;
	/**Modifie le dlai d'attente*/
	virtual void setDelay(PyTime delay)=0;
	/**Retourne l'indice de l'tat de sortie choisi ou -1 sinon*/
	virtual int indOutState()const=0;
	/**Modifie l'indice de l'tat de sortie*/
	virtual void setIndOutState(unsigned ind)=0;
	/**Ajoute une mthode sensible au tirage de la transition*/
	template<typename FctType>void addSensitiveMethod(char const*name,FctType method,int outState=-1){addSensitiveMethod(SSensitiveMethod(parent(),name,static_cast<void(CNamedComp::*)()>(method),outState));}
	void addPySensitiveMethod(char const*name,PyObject*pVoid,int outState=-1);
	/**Supprime une mthode sensible au tirage de la transition*/
	virtual void removeSensitiveMethod(char const*name)=0;
	/**Retourne l'indice de l'tat de sortie atteint lors du tirage de la transition (ou -1 si la transition n'a pas t tire)*/
	virtual int firedState()const=0;
	/**Force  recalculer le dlai d'attente*/
	virtual void invalidate()=0;
	/**Modifie le masque des tats de sortie  monitorer*/
	virtual void setMonitoredOutStateMask(char const*mask)=0;
	/**Retourne le masque des tats de sortie  monitorer*/
	virtual char const*monitoredOutStateMask()const=0;
};

/**Interface de gestion des tats.
	Les tats permettent de construire le diagramme d'tats d'un composant  l'aide des transitions.
	Un tat peut tre actif ou non. Un tat actif active ses transitions de sortie.
	Un tat peut tre export dans une bote de messages comme une vzariable.
*/
class PYC_PUBLIC IState:public virtual IVariable{
protected:
	~IState(){}
public:
	/**Ajoute une transition de sortie de l'tat.*/
	virtual ITransition*addTransition(char const*name)=0;
	/**Retourne vrai si l'tat est actif.*/
	virtual bool isActive()const{return bValue();}
	/**Retourne la date de sortie de l'tat la plus proche (en seconde).*/
	virtual double exitTime()const=0;
	/**Retourne l'automate de l'tat.*/
	virtual IAutomaton*automaton()const=0;
	/**Retourne l'indice de l'tat (doit tre unique dans l'automate).*/
	virtual unsigned int index()const=0;
	/**Retourne la liste des transitions au dpart de l'tat.*/
	virtual std::vector<ITransition*>transitions()const=0;
};

/**Interface de gestion des automates d'tats.
	Les automates grent le diagramme des tats d'un composant.
	Un automate peut restituer l'tat courant ou sont index et tre export dans une bote de messages comme une variable.
*/
class PYC_PUBLIC IAutomaton:public virtual IVariable{
public:
	virtual size_t nbStates()const=0;//Retourne le nombre d'tats de l'automate
	virtual std::vector<IState*>states()const=0;//!<Retourne la liste des tats de l'automate
	/**Modifie l'tat initial de l'automate.*/
	virtual void setInitState(IState*st)=0;
	/**Retourne l'tat initial ou NULL.*/
	virtual IState*initState()const=0;
	/**Retourne l'indice de l'tat courrent ou -1.*/
	virtual int currentIndex()const=0;
	/**Retourne l'tat courant ou NULL.*/
	virtual IState*currentState()const=0;
	/**Ajoute un tat  l'automate.*/
	virtual IState*addState(char const*name,int index)=0;
};

/**Interface des conditions d'tapes (ce n'est jamais un objet python)
*/
struct ICondFct{
	virtual bool operator()()const{return true;}
	virtual~ICondFct(){}
};

class CCondFct:public ICondFct{
public:
	explicit CCondFct(bool (*condition)()):m_Condition(condition){}
	bool (*m_Condition)();
	inline bool operator()()const{
		return m_Condition();
	}
};

/**Interface de manipulation des tapes.*/
class PYC_PUBLIC IStep : public CNamedComp{
protected:
	SBindingM&m_Condition;//!<Dfinition de la condition d'utilisation de l'tape
	bool m_NCond;//!<Si vrai, la condition est utilise  l'envers
	bool m_NCondInit;//!<Si vrai, la condition est initialement utilise  l'envers
	IStep(CSystem&sys, char const*name);
	virtual ~IStep();
	void setCondition(CNamedComp&nc, bool(CNamedComp::*fct)(),bool negate);
public:
	/**Ajoute une mthode  l'tape*/
	virtual void addMethod(CNamedComp const*cmp,char const*methodName)=0;
	/**Renseigne la condition par connexion  une variable ou  un tat.*/
	void setCondition(IVariable&var,bool negate=false);
	void setCondition(IVariable*var,bool negate=false){setCondition(*var,negate);}
	/**Renseigne la condition par une fonction boolenne.*/
	template<typename Type> void setCondition(Type&nc,char const*name,bool(Type::*fct)(),bool negate=false){setCondition(nc, static_cast<bool(CNamedComp::*)()>(fct),negate);}
	/**Renseigne la condition par une fonction Python.*/
	void setPyCondition(CNamedComp&nc, char const*name,PyObject*pyObj,bool negate=false);
	/**Renseigne la condition par une mthode Python nomme.*/
	void setPyCondition(CNamedComp&nc, char const*name,bool negate=false){setPyCondition(nc, name,NULL,negate);}
	/**Renseigne la condition par une valeur boolenne.*/
	void setCondition(bool val);
	/**Retourne la valeur actuelle de la condition.*/
	bool getCondition()const;
};

struct TSchemaType{
	/**Enumr de choix de l'algorithme d'intgration.
	*/
	typedef enum ESchemaType{
		euler=1,//!<Algorithme d'euler
		runge_kutta4,//!<Runge kutta d'ordre 4
		runge_kutta_cash_karp54,//!<Runge kutta d'ordre 4/5
		runge_kutta_dopri5,//!<Runge kutta d'ordre 5
		runge_kutta_fehlberg78,//!<Runge kutta fehlberg
		modified_midpoint,//!<Point milieu modifi
		controlled_rk_cash_karp54,//!<Runge kutta  pas variable
		controlled_rk_dopri5,//!<Runge kutta  pas variable
		controlled_rk_fehlberg78,//!<Runge kutta  pas variable
		controlled_bulirsch_stoer,//!<Bulirsch Stoer naturellement  pas variable
		dense_rk_dopri5,//!<Runge kutta dense et  pas variable
		dense_bulirsch_stoer//!<Bulirsch Stoer dense et  pas variable
	}ESchemaType;
	static char const*st_Names[];
};
/**Interface de gestion des solveurs ODE de Pycatshoo*/
class PYC_PUBLIC IPDMPManager:public CNamedComp
{
protected:
	IPDMPManager(CSystem&system,char const*name):CNamedComp(system,name),m_Trace(0){}
	typedef struct TODEFct:CMethod{
		int m_Order;
		TODEFct():m_Order(0){}
		TODEFct(CNamedComp*comp,char const*name,PyObject*pVoid,int order):CMethod(comp,name,pVoid),m_Order(order){}
		TODEFct(CNamedComp*comp,char const*name,void(CNamedComp::*ODEFct)(),int order):CMethod(comp,name,ODEFct),m_Order(order){}
	}TODEFct;

	virtual void addEquationMethod(TODEFct const&fct)=0;//!<Ajoute une fonction de calcul des drives des variables
	virtual void addBeginMethod(CMethod const&fct)=0;//!<Ajoute une fonction  appeler avant l'intgration
	virtual void addEndMethod(CMethod const&fct)=0;//!<Ajoute une fonction  appeler aprs l'intgration
	virtual void addCondition(CMethod const&fct)=0;//!<Ajoute une fonction de calcul de condition d'arrt de l'intgration
	virtual void addCondition(CTransition*tr)=0;//!<Ajoute une transition utilise pour sa condition
	int m_Trace;
public:
	/**Modifie l'algorithme d'intgration (par dfaut Runge kutta d'ordre 4).*/
	virtual void setSchema(TSchemaType::ESchemaType schema)=0;
	/**Retourne l'algorithme d'intgration utilis.*/
	virtual TSchemaType::ESchemaType schema()const=0;
	/**Modifie le pas d'intgration (par dfaut 0.01).*/
	virtual void setDt(double dt)=0;
	virtual double getDt()const=0;
	/**Modifie le pas d'intgration minimum (par dfaut 0).*/
	virtual void setDtMin(double dtMin)=0;
	virtual double getDtMin()const=0;
	/**Modifie le pas d'intgration maximum (par dfaut l'infini).*/
	virtual void setDtMax(double dtMax)=0;
	virtual double getDtMax()const=0;
	/**Modifie la prcision de recherche des intersections (par dfaut 0.001).*/
	virtual void setDtCond(double dt)=0;
	virtual double getDtCond()const=0;
	/**Modifie la priodicit de mmorisation des variables (par dfaut 0.1).*/
	virtual void setDtMem(double dt)=0;
	virtual double getDtMem()const=0;
	/**Modifie l'option de stockage systmatique des variables (par dfaut false).*/
	virtual void setAlwaysMem(bool bAlways)=0;
	virtual bool alwaysMem()const=0;
	/**Ajoute une variable  grer par le systme d'ODE.*/
	virtual void addODEVariable(IVariable&var)=0;
	/**Ajoute une variable  calculer en mme temps que le systme d'ODE.*/
	virtual void addExplicitVariable(IVariable&var)=0;
	/**Ajoute une fonction  appeler avant l'intgration*/
	template<typename Type> void addBeginMethod(char const*name,Type&comp,void(Type::*beginFct)()){addBeginMethod(CMethod(&comp,name,static_cast<void(CNamedComp::*)()>(beginFct)));}
	/**Ajoute une fonction  appeler aprs l'intgration*/
	template<typename Type> void addEndMethod(char const*name,Type&comp,void(Type::*endFct)(char const*)){addEndMethod(CMethod(&comp,name,static_cast<void(CNamedComp::*)(char const*)>(endFct)));}
	/**Ajoute une fonction d'quation diffrentielle*/
	template<typename Type> void addEquationMethod(char const*name,Type&comp,void(Type::*odeFct)(),int order=0){addEquationMethod(TODEFct(&comp,name,static_cast<void(CNamedComp::*)()>(odeFct),order));}
	/**Ajoute une fonction de calcul de condition*/
	template<typename Type> void addCondition(char const*name,Type&comp,double(Type::*condFct)()){addCondition(CMethod(&comp,name,static_cast<double(CNamedComp::*)()>(condFct)));}
	template<typename Type> void addBoundaryCheckerMethod(char const*name,Type&comp,double(Type::*condFct)()){addCondition(CMethod(&comp,name,static_cast<double(CNamedComp::*)()>(condFct)));}
	/**Ajoute une transition utilise pour sa condition*/
	void addWatchedTransition(ITransition&tr);
	void addPyBeginMethod(char const*name,CComponent&comp,PyObject*pVoid);
	void addPyBeginMethod(char const*name,CComponent&comp){addPyBeginMethod(name,comp,NULL);}
	void addPyEndMethod(char const*name,CComponent&comp,PyObject*pVoid);
	void addPyEndMethod(char const*name,CComponent&comp){addPyEndMethod(name,comp,NULL);}
	void addPyEquationMethod(char const*name,CComponent&comp,PyObject*pVoid,int order=0);
	void addPyEquationMethod(char const*name,CComponent&comp,int order=0){addPyEquationMethod(name,comp,NULL,order);}
	void addPyCondition(char const*name,CComponent&comp,PyObject*pVoid);
	void addPyCondition(char const*name,CComponent&comp){addPyCondition(name,comp,NULL);}
	/**Modifie la trace du PDMP (juste 0 ou non nul)*/
	void setTrace(int trace){m_Trace=trace;}
	int trace()const{return m_Trace;}
	/**Retourne vrai si la variable fait partie des variable explicites du PDMP*/
	virtual bool isExplicitVariable(IVariable const*var)const=0;
	/**Retourne vrai si la variable fait partie des variable d'ODE du PDMP*/
	virtual bool isODEVariable(IVariable const*var)const=0;
};

/**Classe de dfinition d'une quation*/
class PYC_PUBLIC IEquation{
protected:
	virtual~IEquation(){}
public:
	/**Ajoute un coefficient associ  une variable*/
	virtual void setCoefficient(IVariable*var,double cR,double cI=0)=0;
	/**Dfinit la constate du second membre*/
	virtual void setConstant(double cR,double cI=0)=0;
};

/**Classe de dfinition et de rsolution de systmes d'quations linaires.
*/
class PYC_PUBLIC ISLEManager:public CNamedComp{
protected:
	ISLEManager(CSystem&system,char const*name):CNamedComp(system,name){}
	~ISLEManager(){}
public:
	virtual void addEquation(char const*name,CNamedComp&comp,void(CNamedComp::*equation)(IEquation&equ))=0;
	/**Ajoute une quation*/
	template<typename Type>void addEquation(char const*name,Type&comp,void(Type::*equation)(IEquation&equation)){addEquation(name,comp,static_cast<void(CNamedComp::*)(IEquation&equation)>(equation));}
	virtual void addPyEquation(char const*name,CNamedComp&comp,PyObject*pVoid)=0;//Composant ncessaire lors de la destruction d'un composant
	void addPyEquation(char const*name,CNamedComp&comp){addPyEquation(name,comp,NULL);}
	/**Ajoute une variable  calculer par le systme d'quations.*/
	virtual void addVariable(IVariable&var)=0;
	/**Demande la rsolution du systme (retourne VRAI si la rsolution s'est bien passe)*/
	virtual bool solve()=0;
	/**Demande l'impression du problme*/
	virtual void print(char const*fileName)const=0;
};

class PYC_PUBLIC IInequality{
protected:
	virtual~IInequality(){}
public:
	/**Ajoute un coefficient associ  une variable*/
	virtual void setCoefficient(IVariable*var,double cR)=0;
	/**Dfinit le min*/
	virtual void setMin(double Min)=0;
	/**Dfinit le max*/
	virtual void setMax(double Max)=0;
	/**Dfinit les limites min et max*/
	virtual void setLimits(double Max,double Min){setMin(Min),setMax(Max);}
};

struct TMILPMsgLevel{
	typedef enum EMILPMsgLevel{
		off=0,
		error,
		on,
		all
	}EMILPMsgLevel;
};
struct TMILPScalingType{
	typedef enum EMILPScalingType{
		geom_mean=1,
		equilib=16,
		near_p2=32,
		none=64,
		automatic=128
	}EMILPScalingType;
};
struct TMILPAlgorithmType{
	typedef enum EMILPAlgorithmType{
		simplex=0,
		interior_point,
		milp
	}EMILPAlgorithmType;
};
struct TMILPOptionsType{
	typedef enum EMILPOptionsType{
		none=0,
		pr_steepest_edge=1,
		hr_ratio_test=2,
		presolve=4,
		binarize=8,
		sr_heuristic=16,
		fp_heuristic=32,
		ps_heuristic=64,
		gmi_cuts=128,
		mir_cuts=256,
		cov_cuts=512,
		clq_cuts=1024
	}EMILPOptionsType;
};
struct TMILPSimplexType{
	typedef enum EMILPSimplexType{
		primal=0,
		dual,
		dual_primal
	}EMILPSimplexType;
};
struct TMILPIntPointType{
	typedef enum EMILPIntPointType{
		none=0,
		quot_min_degree,
		appr_min_degree,
		sym_appr_min_degree
	}EMILPIntPointType;
};
struct TMILPBranchingType{
	typedef enum EMILPBranchingType{
		first_frac_variable=0,
		last_frac_variable,
		most_frac_variable,
		DrTom_heuristic,
		pseudo_cost_heuristic
	}EMILPBranchingType;
};
struct TMILPBacktrackingType{
	typedef enum EMILPBacktrackingType{
		depth_first=0,
		breadth_first,
		best_local_bound,
		best_projection
	}EMILPBacktrackingType;
};
struct TMILPPreprocessingType{
	typedef enum EMILPPreprocessingType{
		none=0,
		root,
		all
	}EMILPPreprocessingType;
};
/**Classe de dfinition d'un systme mixte entier linaire.
La classe constitue une interface avec la bilbiothque glpk dont elle reprent la plupart des options.
*/
class PYC_PUBLIC IMILPManager:public CNamedComp{
protected:
	bool m_bMaximize;//Sens d'optimisation
	TMILPMsgLevel::EMILPMsgLevel m_MsgLevel;
	int m_MsgPeriod,m_MsgDelay,m_Scaling,m_TimeLimit,m_IterLimit;
	TMILPAlgorithmType::EMILPAlgorithmType m_Algo;
	int m_Options;
	TMILPSimplexType::EMILPSimplexType m_Simplex;
	TMILPIntPointType::EMILPIntPointType m_IntPoint;
	TMILPBranchingType::EMILPBranchingType m_Branching;
	TMILPBacktrackingType::EMILPBacktrackingType m_Backtracking;
	TMILPPreprocessingType::EMILPPreprocessingType m_Preprocessing;
	double m_ObjectiveVal;
	virtual void addIneq(char const*name,CNamedComp&comp,void(CNamedComp::*inequality)(IInequality&equ))=0;
	IMILPManager(CSystem&system,char const*name):CNamedComp(system,name),
		m_bMaximize(false),m_MsgLevel(TMILPMsgLevel::error),m_MsgPeriod(500),m_MsgDelay(0),m_Scaling(TMILPScalingType::none),m_TimeLimit(INT_MAX),m_IterLimit(INT_MAX),m_Algo(TMILPAlgorithmType::milp),m_Options(TMILPOptionsType::pr_steepest_edge|TMILPOptionsType::hr_ratio_test|TMILPOptionsType::presolve),m_Simplex(TMILPSimplexType::primal),m_IntPoint(TMILPIntPointType::appr_min_degree),m_Branching(TMILPBranchingType::DrTom_heuristic),m_Backtracking(TMILPBacktrackingType::best_local_bound),m_Preprocessing(TMILPPreprocessingType::all),m_ObjectiveVal(0){}
public:
	/**Ajoute une inquation*/
	template<typename Type>void addInequality(char const*name,Type&comp,void(Type::*inequality)(IInequality&inequality)){addIneq(name,comp,static_cast<void(CNamedComp::*)(IInequality&inequality)>(inequality));}
	virtual void addPyInequality(char const*name,CNamedComp&comp,PyObject*pVoid)=0;//Composant ncessaire lors de la destruction d'un composant
	void addPyInequality(char const*name,CNamedComp&comp){addPyInequality(name,comp,NULL);}
	virtual void removeInequality(CNamedComp&comp,char const*name)=0;
	/**Ajoute une variable  calculer par le systme d'quations et son coefficient de fonction objectif.*/
	virtual void addVariable(IVariable&var,double co)=0;
	void addVariable(IVariable&var){addVariable(var,0);}
	/**Supprime la variable du systme*/
	virtual void removeVariable(IVariable&var)=0;
	/**Dfinit un coefficient de la fonction objectif*/
	virtual void setVariableObjCoef(IVariable&var,double co)=0;
	/**Dfinit les bornes min et max de la variable*/
	virtual void setVariableLimits(IVariable&var,double min,double max)=0;
	/**Dfinit la borne max de la variable*/
	virtual void setVariableMax(IVariable&var,double max)=0;
	/**Dfinit la borne min de la variable*/
	virtual void setVariableMin(IVariable&var,double min)=0;
	/**Dsigne la variable de stockage de la valeur de la fonction objectif*/
	virtual void setObjectiveVar(IVariable*var)=0;
	/**Dfinit le sens de l'optimisation*/
	void setMaximize(bool bMax){m_bMaximize=bMax;};
	bool isMaximize()const{return m_bMaximize;}
	/**Dfinit l'algorithme utilis parmi alg_simplex, alg_int_point, alg_milp*/
	void setAlgorithm(TMILPAlgorithmType::EMILPAlgorithmType algo){m_Algo=algo;}
	TMILPAlgorithmType::EMILPAlgorithmType algorithm()const{return m_Algo;}
	void setMsgOptions(TMILPMsgLevel::EMILPMsgLevel level,int period,int delay){m_MsgLevel=level;m_MsgPeriod=period;m_MsgDelay=delay;}
	TMILPMsgLevel::EMILPMsgLevel msgLevel()const{return m_MsgLevel;}
	int msgDelay()const{return m_MsgDelay;}
	int msgPeriod()const{return m_MsgPeriod;}
	void setScaling(int sc){m_Scaling=sc;}
	int scaling()const{return m_Scaling;}
	void setOptions(int opts){m_Options=opts;}
	int options()const{return m_Options;}
	void setOption(TMILPOptionsType::EMILPOptionsType opt,bool bSet){if(bSet)m_Options=m_Options|opt;else m_Options=m_Options&~opt;}
	bool option(TMILPOptionsType::EMILPOptionsType opt)const{return (opt&m_Options)!=0;}
	void addOptions(int opts){m_Options=m_Options|opts;}
	void delOptions(int opts){m_Options=m_Options&~opts;}
	void setTimeLimit(int ms){m_TimeLimit=ms;}
	int timeLimit()const{return m_TimeLimit;}
	void setIterLimit(int its){m_IterLimit=its;}
	int iterLimit()const{return m_IterLimit;}
	void setSimplexOpt(TMILPSimplexType::EMILPSimplexType smp){m_Simplex=smp;}
	TMILPSimplexType::EMILPSimplexType simplexOpt()const{return m_Simplex;}
	void setIntPointOpt(TMILPIntPointType::EMILPIntPointType ip){m_IntPoint=ip;}
	TMILPIntPointType::EMILPIntPointType intPointOpt()const{return m_IntPoint;}
	void setBacktracking(TMILPBacktrackingType::EMILPBacktrackingType bt){m_Backtracking=bt;}
	TMILPBacktrackingType::EMILPBacktrackingType backtracking()const{return m_Backtracking;}
	void setBranching(TMILPBranchingType::EMILPBranchingType br){m_Branching=br;}
	TMILPBranchingType::EMILPBranchingType branching()const{return m_Branching;}
	void setPreprocessing(TMILPPreprocessingType::EMILPPreprocessingType pre){m_Preprocessing=pre;}
	TMILPPreprocessingType::EMILPPreprocessingType preprocessing()const{return m_Preprocessing;}
	/**Retourne la valeur de la fonction objectif obtenue*/
	double objectiveValue()const{return m_ObjectiveVal;}
	/**Demande la rsolution du systme (retourne VRAI si la rsolution s'est bien passe)*/
	virtual bool solve()=0;
	/**Demande l'impression du problme au format mps, glp, lp suivant l'extension du fichier*/
	virtual void print(char const*fileName)const=0;
};

/**Classe de stockage d'un tat du systme.
Un objet de cette classe peut tre obtenu en appelant la methode systemState de la classe CSystem.
*/
class PYC_PUBLIC ISystemState{
	ISystemState(ISystemState const&);
	ISystemState const&operator=(ISystemState const&);
protected:
	ISystemState(){}
	virtual ~ISystemState(){}
public :
	virtual void reset()const=0;//!<Replace le systme dans l'tat mmoris
	virtual void release()const=0;//!<Oublie la mmorisation de l'tat
};

struct TOptimAlgorithmType{
	typedef enum EOptimAlgorithmType{
		unknown=0,
		powell=1,
		gradient
	}EOptimAlgorithmType;
	static char const*st_Names[3];
};

/**Classe de calcul d'un tirage prfrentiel.
La classe permet de paramtrer et de lancer les calculs avec ou sans optimisation.
Le calcul est effectu en utilisant le nombre de squences donn pour effectuer une simulation standard.
Le tirage de nouvelles squences est stopp ds que le nombre d'vnements maximum est atteint (s'il est diffrent de 0).

Les fonctions de modification des paramtres de loi peuvent tre dfinies de 2 faons :
- en fournissant une mthode d'une classe fille de la classe CSystem qui ne doit se baser que sur les automates des composants.
- en donnant le numro du paramtre alpha utilis pour le calcul par dfaut : exp(-alpha*nbCompToBreak()*nbCompToBreak())

Les coeffients alpha doivent tre crs grce  addAlpha. Il peuvent tre limits grce  setAlphaMin et setAlphaMax.

La mthode de calcul de nbCompToBreak peut tre paramtre en dfinissant les tats dgrads grce  la mthode addKoState.
Les diffrents tats identifis par un appel de addKoState sont considrs comme un mme tat dgrad ce qui permet d'unifier la perte d'une file.
La mthode nbCompToBreak peut tre surcharge dans la class CSystem.

Les coefficients de correction appliqus aux lois instantanes ne peuvent pas dpasser le paramtre gammaModifierMax (pour viter que la probabilit dpasse 1)

Si nbSimuOptim = 0, un unique tirage prrentiel est excut sans optimisation.
Sinon, on effectue nbSimuOptim optimisation tant qu'on ne satisfait pas la limite de prcision requise sur les alphas.

Si un fichier sequenceFile est fourni, un fichier sequenceFileN est restitu aprs chaque optimisation et un fichier sequenceFile est restitu en fin de calcul.

Le rsultat est obtenu par appel de reliability() et stdDevReliability();
*/
class IImportanceSampling:public CNamedComp{
protected:
	size_t m_NbSimuOptim;//!<Nombre de simulations d'optimisation
	size_t m_NbEvtMax;//!<Nombre maximal d'vnements pour un ensemble de traj constituant une simu
	unsigned int m_Trace;
	double m_GammaModifMax;//!<Maximum admis pour les coefficients de modification des gamma
	double m_CEPrecision;//!<Prcision de maximisation de l'entropie croise
	double m_Precision;//!<Prcision de calcul des alphas
	double m_Reliability,m_StdDevReliability;
	TOptimAlgorithmType::EOptimAlgorithmType m_OptimAlgorithm;
	IImportanceSampling(CSystem&sys,char const*name):CNamedComp(sys,name),m_NbSimuOptim(0),m_NbEvtMax(100),m_Trace(0),m_GammaModifMax(HUGE_VAL),m_CEPrecision(0),m_Precision(0),m_Reliability(0),m_StdDevReliability(0),m_OptimAlgorithm(TOptimAlgorithmType::powell){}
	virtual ~IImportanceSampling(){}
	virtual void addUFunction(char const*transName,char const*uFctName,double (CNamedComp::*)())=0;
public:
	void setTrace(int level){m_Trace=level;}//!<Modifie le niveau de trace
	int trace()const{return m_Trace;}//!<Retourne le niveau de trace
	void setNbSimuOptim(size_t nb){m_NbSimuOptim=nb;}//!<Modifie le nombre d'optimisations  effectuer
	size_t nbSimuOptim()const{return m_NbSimuOptim;}//!<Retourne le nombre d'optimisations  effectuer
	void setNbEvtMax(size_t nb){m_NbEvtMax=nb;}//!<Modifie le nombre maximum d'vnements  obtenir dans les squences d'optimisation
	size_t nbEvtMax()const{return m_NbEvtMax;}//!<Retourne le nombre maximum d'vnements  obtenir dans les squences d'optimisation
	void setCEPrecision(double prec){m_CEPrecision=prec;}//!<Modifie la prcision de maximisation de l'entropie croise
	double CEPrecision()const{return m_CEPrecision;}//!<Retourne la prcision de maximisation de l'entropie croise
	void setAlphaPrecision(double prec){m_Precision=prec;}//!<Modifie la prcision de calcul des Alphas
	double alphaPrecision()const{return m_Precision;}//!<Retourne la prcision de calcul des Alphas
	double gammaModifierMax()const{return m_GammaModifMax;}//!<Retourne le max des coefficients de modification des gammas
	void setGammaModifierMax(double value){m_GammaModifMax=value;}//!<Modifie la limite suprieure des coefficients de modification des gammas
	TOptimAlgorithmType::EOptimAlgorithmType optimAlgorithm()const{return m_OptimAlgorithm;}//!<Retourne l'algorithme utilis pour l'optimisation
	void setOptimAlgorithm(TOptimAlgorithmType::EOptimAlgorithmType type){m_OptimAlgorithm=type;}
	virtual size_t dimension()const=0;//!<Retourne la dimension du vecteur des alphas
	virtual double alphaInit(size_t i)const=0;//!<Retourne le ime alpha valeur initiale
	virtual double alpha(size_t i)const=0;//!<Retourne le ime alpha
	virtual double alphaMax(size_t i)const=0;//!<Retourne le max du ime alpha
	virtual double alphaMin(size_t i)const=0;//!<Retourne le min du ime alpha
	virtual void setAlpha(size_t i,double alpha)=0;//!<Modifie le ime alpha
	virtual void setAlphaMax(size_t i,double value)=0;//!<Modifie la limite suprieure du ime alpha
	virtual void setAlphaMin(size_t i,double value)=0;//!<Modifie la limite infrieure du ime alpha
	virtual void addAlpha(double value)=0;//!<Ajoute un alpha au systme (augmente la dimension de 1)
	virtual void removeAlpha(size_t i)=0;//!<Supprime le ime alpha du systme (diminue la dimension de 1)
	virtual void setSequenceFile(char const*fileName)=0;//!<Modifie le nom de base des fichiers de restitution des optimisations successives
	virtual char const*sequenceFile()const=0;//!<Retourne le nom de base des fichiers de restitution des optimisations successives
	/**Ajoute une fonction de calcul des paramtres en fonction des alphas*/
	template<typename TYPE>void addUFunction(char const*transName,char const*uFctName,TYPE uFct){addUFunction(transName,uFctName,static_cast<double (CSystem::*)()>(uFct));}
	virtual void addUFunction(char const*transName,char const*uFctName,PyObject*uFct)=0;//!<Ajoute une fonction de clacul des paramtres (python)
	virtual void addUFunction(char const*transName,char const*uFctName){addUFunction(transName,uFctName,(PyObject*)NULL);}//!<Ajoute une fonction de clacul des paramtres (python)
	/**Ajoute une utilisation de USimple pour le iAlpha me alpha*/
	virtual void addUFunction(char const*transName,size_t iAlpha)=0;
	virtual void addKOState(char const*name)=0;//!<Dsigne les tats quivalents de perte d'une file
	virtual void removeKOState(char const*name)=0;//!<Supprime un tat quivalent de perte de file
	virtual bool evalReliability()=0;//!<Evalue la fiabilit et son cart type en utilisant les alphas courants ou aprs cross entropy si nbSimuOptim>0
	double reliability()const{return m_Reliability;}
	double stdDevReliability()const{return m_StdDevReliability;}
};

/**Interface d'enregistrement d'une BdC*/
class PYC_PUBLIC IBdC
{
	char const*m_Name;
protected:
	explicit IBdC(char const*name);
public:
	virtual~IBdC();
	/**Ajoute un composant de classe clName et de nom name dans le systme CSystem*/
	virtual CComponent*newComponent(char const*clName,char const*name,CSystem&system)const=0;
	/**Retourne la liste des classes disponibles*/
	virtual std::vector<std::string>classes()const{return std::vector<std::string>();}
	/**Retourne la version de la BdC*/
	virtual char const*version()const{return "";}
};

#endif
