/*************************************
*         Copyright 2015 EDF         *
*************************************/
#ifndef SYSTEME_H
#define SYSTEME_H

#include "Interfaces.h"

class CXMLNode;
class CSequence;
class IRNG;
class CSystemP;
class CVariableDbl;
class CReference;
class CIndicator;

struct TSysMethod{
	typedef enum ESysMethod{
		check,
		start,
		pre_sensib,
		in_sensib,
		post_sensib
	}ESysMethod;
};
struct TSysTrace{
	typedef enum ESysTrace{
		none=0,
		seq_number=1,
		planning=2,
		sys_methods=4,
		sens_methods=8,
		other=1<<30,
		traces=other-1
	}ESysTrace;
};
/**Classe de gestion d'un systme  simuler.
	Cette classe doit comprendre tous les composants de haut niveau du systme  simuler.

	Le principe d'utilisation consiste  sous classer CSystem et  surcharger le constructeur pour y construire les composants
	du systme et les liens entre eux.

	La classe propose des outils de monitoring et de trace.

	La classe permet de dfinir des cibles  travers des fonctions boolennes. Les cibles servent de condition d'arrt des squences.

	La classe permet enfin de grer le droulement de la simulation (initialisation, lancement, redmarrage, gestion des rsultats).
*/
class PYC_PUBLIC CSystem : public IMonitor,public CNamedComp
{
	friend class CSystemP;
public :
	typedef enum TSimuMode{
		m_stopped=0,
		m_interactive,
		m_interactive_p,
		m_replay,
		m_imp_smp,
		m_standard}TSimuMode;
private:
	CSystemP&m_SystemP;
	float m_PctSimu;
	static void*m_BdCPythonLib;
	void doMonitor(IMonitored const&elt)override;
	void monitor(IMonitored&elt,bool bMonit)override;
protected:
	PyObject*m_Self;
	ITarget*addTarget(char const*name,bool (CSystem::*)(),PyObject*pyObj);
	CIndicator*addIndicator(char const*name,float (CSystem::*)(),PyObject*pyObj);
	void addSystemMethod(TSysMethod::ESysMethod type,CMethod fct);
	void deleteData();
public:
	/**Construit un systme de nom name.*/
	explicit CSystem(char const*name);
	CSystem(PyObject*pyObj,char const*name);
	virtual~CSystem(void);
	void deleteSys();//!<Dtruit le systme sauf si m_Self est non nul (alors, ne dtruit que l'intrieur)
	/**Retourne le systme en cours de simulation*/
	static CSystem*glSystem();
	CSystemP&systemP(){return m_SystemP;}
	CSystemP const&systemP()const{return m_SystemP;}
	/**Enregistre une BdC pour les systmes  construire*/
	static void registerBdC(char const*name,IBdC&BdC);
	/**Desenregistre une BdC pour les systmes  construire*/
	static void unregisterBdC(char const*name);
	/**Retourne la liste des noms des BdC*/
	static std::vector<std::string>BdCs();
	/**Retourne la BdC de nom name ou NULL*/
	static IBdC const*BdC(char const*name);
	bool verify();//!<Vrifie le systme avant le lancement d'une simulation
	/**Construit un composant de nom name et de classe clName.
	\param name nom du composant
	\param clName nom de la classe du composant
	\param bdcName nom de la BdC (facultatif)*/
	CComponent*newComponent(char const*name, char const*clName,char const*bdcName=NULL);
	std::vector<CComponent*>getComponents(char const*name=NULL, char const*className=NULL)const;//!<Retourne tous les composants de nom name (# en tte si regex, tous rcursivement si NULL) et de classe className (# en tte si regex)
	std::vector<IPDMPManager*>getPDMPManagers(char const*name=NULL)const;//!<Retourne tous les PDMPManagers correspondant  l'expression rgulire name
	std::vector<ISLEManager*>getSLEManagers(char const*name=NULL)const;//!<Retourne tous les SLEManagers correspondant  l'expression rgulire name
	std::vector<IMILPManager*>getMILPManagers(char const*name=NULL)const;//!<Retourne tous les MILPManagers correspondant  l'expression rgulire name
	std::vector<ITransition*>getTransitions(char const*name=NULL,char const*className=NULL)const;//!<Retourne toutes les transitions de nom name (# en tte si regex) et dont le composant est de classe className (# en tte si regex)
	std::vector<IVariable*>getVariables(char const*name=NULL,char const*className=NULL)const;//!<Retourne toutes les variables de nom name (# en tte si regex) et dont le composant est de classe className (# en tte si regex)
	std::vector<IReference*>getReferences(char const*name=NULL,char const*className=NULL)const;//!<Retourne toutes les rfrences de nom name (# en tte si regex) et dont le composant est de classe className (# en tte si regex)
	std::vector<IState*>getStates(char const*name=NULL,char const*className=NULL)const;//!<Retourne tous les tats de nom name (# en tte si regex) et dont le composant est de classe className (# en tte si regex)
	std::vector<IAutomaton*>getAutomatons(char const*name=NULL,char const*className=NULL)const;//!<Retourne tous les automates de nom name (# en tte si regex) et dont le composant est de classe className (# en tte si regex)
	std::vector<IIndicator*>getIndicators(char const*name=NULL)const;//!<Retourne tous les indicateurs de nom name (# en tte si regex)
	std::vector<ITarget*>getTargets(char const*name)const;//!<Retourne toutes les cibles de nom name (# en tte si regex)
	std::vector<IStep*>getSteps(char const*name=NULL)const;//!<Retourne toutes les tapes de nom name (# en tte si regex)
	std::vector<IMonitored const*>getMonitoredElts(char const*name=NULL)const;//!<Retourne tous les lments monitors correspondant  l'expression rgulire name
	std::vector<ITransition*>getPlannedTransitions()const;//!<Retourne toutes les transitions actives
	std::vector<ITransition*>getActiveTransitions()const;//!<Retourne toutes les transitions actives
	std::vector<ITransition*>getActiveTransitionsDyn()const;//!<Retourne toutes les transitions dont l'tat de dpart est actif et la condition actuelle est vrai
	std::vector<ITransition*>getFiredTransitions()const;//!Retourne les transitions tires au pas prcdent
	IMessageBox*getMessageBox(char const*name)const;//!<Retourne la boite de message de nom complet name
	ISystemState const*systemState()const;//!<Retourne un stockage de l'tat du systme
	/**Ajoute une mthode systme de type type et de nom name*/
	template<typename Type>void addSystemMethod(TSysMethod::ESysMethod type,Type&nc,char const*name,void(Type::*method)()){addSystemMethod(type,CMethod(&nc,name,static_cast<void(CNamedComp::*)()>(method)));}
	void addPySystemMethod(TSysMethod::ESysMethod type,CNamedComp&nc,char const*name,PyObject*pVoid){addSystemMethod(type,CMethod(&nc,name,pVoid));}
	/**Supprime une mthode systme de type type et de nom name*/
	void removeSystemMethod(TSysMethod::ESysMethod type,CNamedComp&comp,char const*name);
	/**Ajoute un indicateur et dfinit sa fonction de calcul.
	\param name nom de l'indicateur
	\param fct fonction indicatrice*/
	template<typename TYPE>IIndicator*addIndicator(char const*name,float (TYPE::*fct)()){
		return (IIndicator*)addIndicator(name,static_cast<float (CSystem::*)()>(fct),NULL);
	}
	/**Ajoute un indicateur dfini par une variable et ventuellement une comparaison avec une valeur.
	\param name nom de l'indicateur
	\param elementName nom de la variable
	\param elementType type de l'lment (ST VAR AUT)
	\param op oprateur de comparaison
	\param value valeur de comparaison*/
	IIndicator*addIndicator(char const*name, char const * elementName, char const * elementType, char const * op=NULL, float value=0);
	IIndicator*addIndicator(char const*name,IVariable&var, char const * op=NULL, float value=0);
	/**Ajoute un indicateur dfini par une fonction python*/
	IIndicator*addPyIndicator(char const*name,PyObject*pyObj);
	/**Retourne l'indicateur de nom name.*/
	IIndicator*indicator(char const*name)const;
	/**Supprime l'indicateur de nom name.*/
	void removeIndicator(char const*name);
	/**Ajoute un instant de mmorisation des indicateurs.*/
	void addInstant(double date);
	/**Ajoute un instant de mmorisation des indicateurs.*/
	void addInstants(double minDate, double maxDate, int nTotalPoints);
	/**Retourne les instants de mmorisation des indicateurs.*/
	std::vector<double>instants()const;
	virtual size_t nbCompToBreak();//!<Nombre composants  casser pour atteindre la dfaillance ( surcharger pour utiliser USimple)
	/**Ajoute une cible et dfinit sa fonction condition.
	\param name nom de la cible
	\param fct fonction condition (retourne vrai si la cible est atteinte)*/
	template<typename TYPE>ITarget*addTarget(char const*name,bool(TYPE::*fct)()){
		return addTarget(name,static_cast<bool(CSystem::*)()>(fct),NULL);
	}
	ITarget*addPyTarget(char const*name,PyObject*pyObj);
	ITarget*addTarget(char const*name,IVariable&var,char const*op="!=",float value=0);
	ITarget*addTarget (char const * name, char const * elementName, char const * elementType, char const*op="!=", float value=0);
	/**Suprime la cible de nom name.*/
	void removeTarget(char const*name);
	/**Retourne la cible de nom name.*/
	ITarget*target(char const*name);
	/**Ajoute une tape de nom name.
	La mthode retourne l'tape ajoute ou retrouve dans la liste des tapes.*/
	IStep*addStep(char const*name);
	/**Insert une tape de nom name en position ind.
	La mthode retourne l'tape ajoute ou retrouve dans la liste des tapes.*/
	IStep*insertStep(char const*name,unsigned int index);
	/**Retourne l'tape de nom name*/
	IStep*step(char const*name)const;
	/**Modifie la condition de l'tape de nom name*/
	void setStepCondition(char const*name,bool val);
	/**Modifie la condition de l'tape de nom name*/
	void setStepCondition(char const*name,IVariable&var,bool negate=false);
	/**Supprime l'tape de nom name*/
	void removeStep(char const*name);
	/**Ajoute une mthode  l'tape de nom name*/
	void addStepMethod(char const*name,CComponent const*comp, char const*methodName);
	/**Abandonne la construction de la squence*/
	void abortSeq();
	/**Abandonne la simulation*/
	void abortSim();
	/**Retourne une variable d'un type donn*/
	IVariable*getElement(char const*name,char const*type)const;
	/**Retourne le composant de nom name (nom ventuellement compos)
	\param name nom complet du composant recherch
	\param check indicateur de contrle derreur : si vrai, une exception est lance si le composant nexiste pas*/
	CComponent*getComponent(char const*name,bool check=true)const;
	/**Retourne le gestionnaire d'ODE de nom name*/
	IPDMPManager*getPDMPManager(char const*name)const;
	/**Retourne le gestionnaire d'ODE courant*/
	IPDMPManager*currentPDMPManager()const;
	/**Dsigne le gestionnaire d'ODE courant (peut tre null si pas d'ODE)*/
	void setCurrentPDMPManager(char const*name);
	/**Ajoute un nouveau gestionnaire d'ODE (s'il existe dj, retourne celui l)*/
	IPDMPManager*addPDMPManager(char const*name);
	/**Retourne le systme d'quations linaires de nom name*/
	ISLEManager*getSLEManager(char const*name);
	/**Ajoute un nouveau systme d'quations linaires (s'il existe dj, retourne celui l)*/
	ISLEManager*addSLEManager(char const*name);
	/**Retourne l'optimiseur sous contraintes de nom name*/
	IMILPManager*getMILPManager(char const*name);
	/**Ajoute un nouvel optimiseur sous contraintes (s'il existe dj, retourne celui l)*/
	IMILPManager*addMILPManager(char const*name);
	/**Modifie le nom du fichier de rsultats.*/
	void setResultFileName(char const*filename,bool bBin);
	/**Retourne le nom du fichier de rsultats.*/
	char const*resFileName()const;
	/**Retourne vrai si le stockage des squences est binaire*/
	bool binSeqFile()const;
	/**Retourne l'instant courant.*/
	PyTime currentTime()const;
	/**Retourne l'indice de branche courant.*/
	unsigned currentIndBranch()const;
	/**Lance nb simulation jusqu'au temps dateFin (si add vrai on ajoute des squences, sinon on les prolonge)*/
	void simulate();
	/**Rejoue la squence et s'arrte en fin de squence.*/
	void replaySequence(ISequence const*seq);
	/**Dmarre une session interactive*/
	void startInteractiv();
	/**Arrte une session interactive*/
	void stopInteractiv();
	/**Avance jusqu' la date date en interactif FMU*/
	void stepInteractiv(PyTime date);
	/**Modifie la date de tirage et le numro de sortie de la transition*/
	void setTransPlanning(ITransition*trans,PyTime date,unsigned out);
	/**Avance la squence courante jusqu' la prochaine transition ou  la prochaine interruption d'intgration*/
	void stepForward();
	/**Recule  l'tat prcdent*/
	void stepBackward();
	/**Recule  l'tat initial*/
	void restart();
	/**Retourne la squence courante*/
	ISequence const*currentSequence()const;
	/**Met  jour le planning pour tenir compte d'une modification des paramtres des lois*/
	void updatePlanningForParameters();
	/**Connecte deux botes de messages de composants.
	\param c1 composant de dpart
	\param msgB1 bote de messages de c1
	\param c2 composant d'arrive
	\param msgB2 bote de messages de c2
	\param weight poids associ  la connexion (par dfaut 1)*/
	void connect(CComponent&c1,char const*msgB1,CComponent&c2,char const*msgB2,double weight=1);
	/**Connecte deux botes de messages de composants.*/
	void connect(char const*c1Name,char const*msgB1,char const*c2Name,char const*msgB2,double weight=1);
	/**Dconnecte deux botes de messages de composants.*/
	void disconnect(CComponent&c1,char const*msgB1,CComponent&c2,char const*msgB2);
	/**Dconnecte deux botes de messages de composants.*/
	void disconnect(char const*c1Name,char const*msgB1,char const*c2Name,char const*msgB2);
	/**Retourne vrai si la simulation d'une squence est en cours*/
	bool isRunning()const;
	TSimuMode simuMode()const;
	/**Retourne le pourcentage de la simulation dj ralis*/
	float pctSimu()const{return m_PctSimu;}
	/**Retourne vrai si la rsolution d'un systme d'ODE est en cours*/
	bool isODESolving()const;
	/**Retourne vrai si le systme est reconstruit uniquement  partir du fichier de rsultats*/
	bool isAutoBuilt()const;
	/**Supprime les rsultats courants.*/
	void clearRes();
	/**Modifie le nombre de squences  tirer*/
	void setNbSeqToSim(long long nbSeq);
	/**Retourne le nombre de squences  tirer.*/
	SEQ_ID nbSeqToSim()const;
	/**Retourne le nombre de squences tires.*/
	SEQ_ID nbSeqSim()const;
	/**Retourne le nombre de squences conserves.*/
	SEQ_ID nbSequences()const;
	/**Retourne la liste des squences conserves*/
	std::vector<ISequence*>sequences()const;
	/**Retourne la numSeqme squence parmi les nbSequences conserves*/
	ISequence*sequence(SEQ_ID numSeq)const;
	/**Retourne la squence suivante ou la premire si seq est NULL*/
	ISequence*nextSequence(ISequence const*seq)const;
	/**Renseigne la fonction de filtrage des squences sous la forme d'un foncteur boolen d'une squence (la fonction retourne vrai si la squence doit tre conserve)*/
	void setSeqFilter(IFilterFct*seqFilter);
	/**Renseigne la fonction de filtrage des squences sous la forme d'une fonction boolenne d'une squence (la fonction retourne vrai si la squence doit tre conserve).*/
	void setSeqFilter(bool (*seqFilter)(ISequence const&));
	/**Renseigne la fonction de filtrage des squences sous la forme d'un pattern de dfinition d'un filtre*/
	void setSeqFilter(char const*filterPattern);
	/**Retourne l'lment monitor de nom name*/
	IMonitored const*getMonitoredElt(char const*name,char const*type=NULL)const;
	/**Demande l'inhibition des transitions de nom name (le nom peut tre compos).*/
	void inhibateTrans(char const*name,int type,bool inhibate);
	/**Demande l'inhibition de la cible de nom name (toutes si NULL)*/
	void inhibateTarget(char const*name,bool inhibate);
	/**Demande le monitoring du ou des variables de nom name (le nom peut tre compos).*/
	void monitorVariable(char const*name,int level=1);
	/**Demande le monitoring de la ou des transitions de nom name (le nom peut tre compos).*/
	void monitorTransition(char const*name,int level=1,char const*stMask=NULL);
	/**Demande le monitoring du ou des tats de nom name (le nom peut tre compos).*/
	void monitorState(char const*name,int level=1);
	/**Demande le monitoring du ou des automates de nom name (le nom peut tre compos).*/
	void monitorAutomaton(char const*name,int level=1);
	/**Fixe le niveau de trace du ou des variables de nom name (le nom peut tre compos).*/
	void traceVariable(char const*name,int level);
	/**Fixe le niveau de trace du ou des transition de nom name (le nom peut tre compos).*/
	void traceTrans(char const*name,int level);
	/**Fixe le niveau de trace du ou des tats de nom name (le nom peut tre compos).*/
	void traceState(char const*name,int level);
	/**Fixe le niveau de trace du ou des automates de nom name (le nom peut tre compos).*/
	void traceAutomaton(char const*name,int level);
	/**Fixe le niveau de trace du systme.*/
	void setTrace(int level);
	/**Retourne le niveau de trace du systme.*/
	int trace()const;
	/**Demande l'valuation de la vraisemblance*/
	void setEvalLikelyhood(bool bEval);
	/**Retourne vrai si la vraisemblance est value*/
	bool evalLikelyhood()const;
	/**Enregistre les paramtres dans le fichier filename.*/
	void dumpParameters(char const*filename,bool bBin)const;
	void dumpParameters(CXMLNode&root,bool bBin)const;
	/**Charge les paramtres du fichier (si permissive est vrai, on tente de charger le maximum malgr les erreurs)*/
	void loadParameters(char const*filename,bool permissive=false);
	void loadParameters(CXMLNode const&root);
	/**Enregistre les rsultats dans le fichier filename.*/
	void dumpResults(char const*filename,bool bBin)const;
	/**Charge les rsultats  partir du fichier filename.*/
	void loadResults(char const*filename);
	/**Charge un fichier de paramtres ou de rsultats.*/
	void load(char const*filename);
	/**Retourne l'instant de fin des squences*/
	PyTime seqEndTime()const;
	PyTime tMax()const;//!<Retourne la date de fin des squences courantes ou souhaites
	/**Modifie la date de fin de simulation. (si tMax>0 on prolonge les squences du dernier niveau, si tMax=0 on ajout des squences au dernier niveau)*/
	void setTMax(PyTime tMax); 
	/**Modifie la graine du gnrateur de nombres alatoires.*/
	void setRNGSeed(unsigned seed);
	/**Retourne la graine du gnrateur alatoire*/
	unsigned RNGSeed()const;
	/**Modifie le gnrateur alatoire utilis.
	Choix du nom parmi :
	- KISS (gnrateur par dfaut)
	- yarn5 (unique gnrateur utilisable avec MPI)
	- mt19937.*/
	IRNG*setRNG(char const*name);
	/**Retourne le gnrateur alatoire courant.*/
	IRNG*getRNG();
	/**Modifie la taille d'un bloc de gnration de nombre alatoire affect  une squence.
	Pour un bloc de taille nulle, on n'effectue pas de saut d'une squence  l'autre.*/
	void setRNGBlocSize(unsigned int RNGBlocSize);
	/**Retourne la taille du bloc de gnration de nombre alatoire utilise.*/
	unsigned int RNGBlocSize()const;
	/**Retourne le rang de la tche parmi les tches parallles de MPI.*/
	unsigned int MPIRank()const;
	/**Retourne le nombre de tches parallles MPI.*/
	unsigned int MPINbTasks()const;
	/**Demande le chargement d'une BdC python*/
	static bool loadPythonBdC(char const*moduleName);
	/**Demande le chargement d'une bibliothque de BdC*/
	static bool loadBdCLibrary(char const*fileName);
	/**Retourne les messages (y compris les messages python si la BdC est charge avec le chargeur python)*/
	static char const*messages(bool bClear=true);
	/**Dclare un prparamtrage des variables de nom name*/
	void setPreAssignment(char const*name,char const*value);
	/**Retourne la proprit de d'utilisation des squences filtres pour les indicateurs*/
	bool keepFilteredSeqForInd()const;
	/**Modifie la conservation des squences filtres pour le clacul des indicateurs*/
	void setKeepFilteredSeqForInd(bool keep);
	/**Incrmente le compteur de rfrence de l'objet Python (uniquement pour un systme cr en Python)*/
	static PyObject*(*incRef)(PyObject*obj);
	/**Dcrmente le compteur de rfrence de l'objet Python (uniquement pour un systme cr en Python)*/
	static void(*decRef)(PyObject*obj);
	/**Retourne vrai si l'objet est une fonction (uniquement pour un systme cr en Python), lance une exception si name est non null*/
	static bool(*isCallable)(PyObject*obj,char const*name);
	/**Retourne le message d'erreur de la dernire exception python*/
	static char const*(*pythonErrorMsg)();
	static void setPythonFct(PyObject*(*incRef)(PyObject*obj),void(*decRef)(PyObject*obj),bool(*isCallable)(PyObject*,char const*),char const*(*pythonErrorMsg)());
	/**Met  jour la liste des variables rinitialises pour la variable*/
	void updateReinitVar(CVariable&var);
	/**Modifie l'option de stockage de la squence dterministe*/
	void setStoreDeterministicSeq(bool store);
	/**Retourne l'option de stockage de la squence dterministe*/
	bool storeDeterministicSeq()const;
	/**Retourne l'outil de paramtrage du tirage prfrentiel*/
	IImportanceSampling*importanceSampling()const;
	/**Dfinit l'emploi du tirage prfrentiel*/
	void setUseImportanceSampling(bool bUse);
};

#endif
